# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Data loader for the Clinc Intent Detection dataset.

Clinc Intent Detection dataset [1] is a text classification task for
intent detection in a task-oriented dialog system. The data covers 150 intent
classes over 10 domains, and contains 150 short (< 30 tokens) spoken queries
for each class. The dataset also contains a large amount of out-of-domain
queries to assess the intent classifier's ability in handling users' request
that is out of the scope of the known services.

Note:
1. The data is split into train-validation-test ratio of roughly 65%:15%:20%,
   and the in-domain (IND) data and the out-of-domain (OOD) data are separated
   into different splits which can be used separately or together.
   Current implementation supports reading only IND data, only OOD data
   or both.

2. The dataset is stored in TFRecord format. It contains both the original
   sentence and intent name, stored in the fields _INTENT_NAME and
   _UTTERANCE_NAME, respectively.
   It also contains the tokenized version of the sentence and the label (stored
   in fields _FEATURE_NAME and _LABEL_NAME) using tf.keras' default tokenizer
   with a vocabulary size of 7292 (which covers all words in the dataset).


## References
[1]: Tony Finch. An Evaluation Dataset for Intent Classification and
     Out-of-Scope Prediction.
     In _Empirical Methods in Natural Language Processing_, 2019.
     https://www.aclweb.org/anthology/D19-1131.pdf
"""
import json
import os.path

from typing import Any, Dict, Tuple

import tensorflow.compat.v2 as tf
from uncertainty_baselines.datasets import base

# filenames for in-domain (IND), out-of-domain (OOD) and combined datasets
_FILENAME_TRAIN_IND = 'train.tfrecord'
_FILENAME_VAL_IND = 'val.tfrecord'
_FILENAME_TEST_IND = 'test.tfrecord'

_FILENAME_TRAIN_OOD = 'train_ood.tfrecord'
_FILENAME_VAL_OOD = 'val_ood.tfrecord'
_FILENAME_TEST_OOD = 'test_ood.tfrecord'

_FILENAME_TRAIN_ALL = 'train_all.tfrecord'
_FILENAME_VAL_ALL = 'val_all.tfrecord'
_FILENAME_TEST_ALL = 'test_all.tfrecord'

_FILENAME_TOKENZIER = 'keras_tokenizer.json'

_NUM_TRAIN_IND = 15000
_NUM_TRAIN_OOD = 100
_NUM_VAL_IND = 3000
_NUM_VAL_OOD = 100
_NUM_TEST_IND = 4500
_NUM_TEST_OOD = 1000

_NUM_TRAIN_ALL = _NUM_TRAIN_IND + _NUM_TRAIN_OOD
_NUM_VAL_ALL = _NUM_VAL_IND + _NUM_VAL_OOD
_NUM_TEST_ALL = _NUM_TEST_IND + _NUM_TEST_OOD

_LABEL_NAME = 'intent_label'
_FEATURE_NAME = 'utterance_indices'
_INTENT_NAME = 'intent_name'
_UTTERANCE_NAME = 'utterance'
_NUM_TOKEN_NAME = 'utterance_num_tokens'

_FEATURE_LENGTH = 32  # Maximum number of tokens per sentence


def _build_dataset(glob_dir: str, is_training: bool) -> tf.data.Dataset:
  cycle_len = 10 if is_training else 1
  dataset = tf.data.Dataset.list_files(glob_dir, shuffle=is_training)
  dataset = dataset.interleave(tf.data.TFRecordDataset, cycle_length=cycle_len)
  return dataset


def _make_features_spec() -> Dict[str, tf.io.FixedLenFeature]:
  return {
      _LABEL_NAME: tf.io.FixedLenFeature([], tf.int64),
      _FEATURE_NAME: tf.io.FixedLenFeature([_FEATURE_LENGTH], tf.int64),
      _INTENT_NAME: tf.io.FixedLenFeature([], tf.string),
      _UTTERANCE_NAME: tf.io.FixedLenFeature([], tf.string),
      _NUM_TOKEN_NAME: tf.io.FixedLenFeature([], tf.int64),
  }


def _load_tokenizer(
    tokenizer_dir: str) -> tf.keras.preprocessing.text.Tokenizer:
  with tf.io.gfile.GFile(tokenizer_dir) as tokenizer_file:
    tokenizer_json = json.load(tokenizer_file)
  return tf.keras.preprocessing.text.tokenizer_from_json(tokenizer_json)


def _get_num_examples_and_filenames(
    data_mode: str) -> Tuple[Dict[str, int], Dict[str, str]]:
  """Retrieves the number of examples and filenames according to data mode."""
  if data_mode == 'ind':
    num_examples = {
        'train': _NUM_TRAIN_IND,
        'val': _NUM_VAL_IND,
        'test': _NUM_TEST_IND
    }
    file_names = {
        'train': _FILENAME_TRAIN_IND,
        'val': _FILENAME_VAL_IND,
        'test': _FILENAME_TEST_IND
    }
  elif data_mode == 'ood':
    num_examples = {
        'train': _NUM_TRAIN_OOD,
        'val': _NUM_VAL_OOD,
        'test': _NUM_TEST_OOD
    }
    file_names = {
        'train': _FILENAME_TRAIN_OOD,
        'val': _FILENAME_VAL_OOD,
        'test': _FILENAME_TEST_OOD
    }
  elif data_mode == 'all':
    num_examples = {
        'train': _NUM_TRAIN_ALL,
        'val': _NUM_VAL_ALL,
        'test': _NUM_TEST_ALL
    }
    file_names = {
        'train': _FILENAME_TRAIN_ALL,
        'val': _FILENAME_VAL_ALL,
        'test': _FILENAME_TEST_ALL
    }
  else:
    raise ValueError('"data_mode" can only be one of "ind", "ood" or "all". '
                     'Got "{}".'.format(data_mode))

  return num_examples, file_names


class ClincIntentDetectionDataset(base.BaseDataset):
  """Clinc Intent Detection dataset builder class."""

  def __init__(self,
               batch_size: int,
               eval_batch_size: int,
               shuffle_buffer_size: int = None,
               num_parallel_parser_calls: int = 64,
               dataset_dir: str = None,
               data_mode: str = 'ind',
               **unused_kwargs: Dict[str, Any]):
    """Initializer.

    Args:
      batch_size: the training batch size.
      eval_batch_size: the validation and test batch size.
      shuffle_buffer_size: the number of example to use in the shuffle buffer
        for tf.data.Dataset.shuffle().
      num_parallel_parser_calls: the number of parallel threads to use while
        preprocessing in tf.data.Dataset.map().
      dataset_dir: directory containing the TFRecord datasets and the tokenizer.
      data_mode: One of ['ind', 'ood', 'all'] to decide whether to read only
        in-domain data, or read only out-of-domain data, or read both.
    """
    self._dataset_dir = dataset_dir
    num_examples, self._file_names = _get_num_examples_and_filenames(data_mode)
    self.tokenizer = _load_tokenizer(
        tokenizer_dir=os.path.join(self._dataset_dir, _FILENAME_TOKENZIER))

    super(ClincIntentDetectionDataset, self).__init__(
        name='clinc_intent',
        num_train_examples=num_examples['train'],
        num_validation_examples=num_examples['val'],
        num_test_examples=num_examples['test'],
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=shuffle_buffer_size,
        num_parallel_parser_calls=num_parallel_parser_calls)

    self.info['num_classes'] = 150
    self.info['feature_size'] = _FEATURE_LENGTH

  def _read_examples(self, split: base.Split) -> tf.data.Dataset:
    if split == base.Split.TRAIN:
      return _build_dataset(
          glob_dir=os.path.join(self._dataset_dir, self._file_names['train']),
          is_training=True)
    if split == base.Split.VAL:
      return _build_dataset(
          glob_dir=os.path.join(self._dataset_dir, self._file_names['val']),
          is_training=False)
    if split == base.Split.TEST:
      return _build_dataset(
          glob_dir=os.path.join(self._dataset_dir, self._file_names['test']),
          is_training=False)

  def _create_process_example_fn(self, split: base.Split) -> base.PreProcessFn:

    def _example_parser(example: tf.train.Example) -> Dict[str, tf.Tensor]:
      """Parse features and labels from a serialized tf.train.Example."""
      features_spec = _make_features_spec()
      features = tf.io.parse_example(example, features_spec)
      labels = tf.cast(features.pop(_LABEL_NAME), tf.int32)
      utterance_indices = features[_FEATURE_NAME]
      num_tokens = features[_NUM_TOKEN_NAME]
      return {'features': utterance_indices, 'labels': labels,
              'num_tokens': num_tokens}

    return _example_parser



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Dataset getter utility."""

import json
import logging
from typing import Any, Dict, List

from uncertainty_baselines.datasets.base import BaseDataset
from uncertainty_baselines.datasets.cifar import Cifar100Dataset
from uncertainty_baselines.datasets.cifar import Cifar10Dataset
from uncertainty_baselines.datasets.clinc_intent import ClincIntentDetectionDataset
from uncertainty_baselines.datasets.criteo import CriteoDataset
from uncertainty_baselines.datasets.glue import GlueDatasets
from uncertainty_baselines.datasets.imagenet import ImageNetDataset
from uncertainty_baselines.datasets.mnist import MnistDataset
from uncertainty_baselines.datasets.mnli import MnliDataset
from uncertainty_baselines.datasets.svhn import SvhnDataset


def get_dataset_names() -> List[str]:
  return [
      'cifar100', 'cifar10', 'clinic_intent', 'criteo', 'imagenet', 'mnist',
      'mnli', 'svhn', 'glue/cola', 'glue/sst2', 'glue/mrpc', 'glue/qqp',
      'glue/qnli', 'glue/rte', 'glue/wnli', 'glue/stsb'
  ]


def get(
    dataset_name: str,
    batch_size: int,
    eval_batch_size: int,
    **hyperparameters: Dict[str, Any]) -> BaseDataset:
  """Gets a dataset builder by name.

  Args:
    dataset_name: Name of the dataset builder class.
    batch_size: the training batch size.
    eval_batch_size: the validation/test batch size.
    **hyperparameters: dict of possible kwargs to be passed to the dataset
      constructor.

  Returns:
    A dataset builder class with a method .build(split) which can be called to
    get the tf.data.Dataset, which has elements that are a dict with keys
    'features' and 'labels'.

  Raises:
    ValueError: If dataset_name is unrecognized.
  """
  logging.info(
      'Building dataset %s with additional kwargs:\n%s',
      dataset_name,
      json.dumps(hyperparameters, indent=2, sort_keys=True))
  if dataset_name == 'cifar100':
    dataset_class = Cifar100Dataset
  elif dataset_name == 'cifar10':
    dataset_class = Cifar10Dataset
  elif dataset_name == 'clinic_intent':
    dataset_class = ClincIntentDetectionDataset
  elif dataset_name == 'criteo':
    dataset_class = CriteoDataset
  elif dataset_name == 'imagenet':
    dataset_class = ImageNetDataset
  elif dataset_name == 'mnist':
    dataset_class = MnistDataset
  elif dataset_name == 'mnli':
    dataset_class = MnliDataset
  elif dataset_name == 'svhn':
    dataset_class = SvhnDataset
  elif 'glue/' in dataset_name:
    dataset_class = GlueDatasets[dataset_name]
  else:
    raise ValueError('Unrecognized dataset name: {!r}'.format(dataset_name))

  return dataset_class(
      batch_size=batch_size,
      eval_batch_size=eval_batch_size,
      **hyperparameters)



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Data loader for the Jigsaw Toxic Comment classification dataset.

A dataset for training toxicity detection systems. It contains comments from
Wikipedia’s talk page edits that contains detecting different types of
toxicity like threats, obscenity, insults, and identity-based hate.

The sentences in this dataset are pre-processed using a standard WordPiece
tokenizer using the standard BERT vocabulary. The original testing dataset
contains records without valid labels, which are not included in this version.

DISCLAIMER: This dataset contains text that may be considered profane, vulgar,
or offensive.

## References

[1]: Betty van Aken, Julian Risch, Ralf Krestel, Alexander Löser.
     Challenges for Toxic Comment Classification: An In-Depth Error Analysis.
     In _Proceedings of the 2nd Workshop on Abusive Language Online_, 2018.
     https://www.aclweb.org/anthology/W18-5105/
"""
import os.path

from typing import Dict

import tensorflow.compat.v2 as tf
from uncertainty_baselines.datasets import base

_FILENAME_TRAIN = 'train.tfrecord'
_FILENAME_TEST = 'test.tfrecord'

_FILENAME_TOKENZIER = 'keras_tokenizer.json'

_NUM_TRAIN = 159571
_NUM_TEST = 63978

_LABEL_BINARY_NAME = 'label_binary_ids'
_LABEL_MULTI_NAME = 'label_ids'

_SENTENCE_ID_NAME = 'input_ids'
_SENTENCE_TOKEN_NAME = 'input_tokens'
_INPUT_MASK_NAME = 'input_mask'
_SEGMENT_ID_NAME = 'segment_ids'

_FEATURE_LENGTH = 512  # Maximum number of tokens per sentence.
_LABEL_LENGTH = 6  # Number of Toxicity categories.


def _build_dataset(glob_dir: str, is_training: bool) -> tf.data.Dataset:
  cycle_len = 10 if is_training else 1
  dataset = tf.data.Dataset.list_files(glob_dir, shuffle=is_training)
  dataset = dataset.interleave(tf.data.TFRecordDataset, cycle_length=cycle_len)
  return dataset


def _make_features_spec() -> Dict[str, tf.io.FixedLenFeature]:
  return {
      _LABEL_BINARY_NAME: tf.io.FixedLenFeature([], tf.int64),
      _LABEL_MULTI_NAME: tf.io.FixedLenFeature([_LABEL_LENGTH], tf.int64),
      _SENTENCE_ID_NAME: tf.io.FixedLenFeature([_FEATURE_LENGTH], tf.int64),
      _SENTENCE_TOKEN_NAME: tf.io.FixedLenFeature([], tf.string),
      _INPUT_MASK_NAME: tf.io.FixedLenFeature([_FEATURE_LENGTH], tf.int64),
      _SEGMENT_ID_NAME: tf.io.FixedLenFeature([_FEATURE_LENGTH], tf.int64),
  }


class JigsawToxicCommentDataset(base.BaseDataset):
  """Clinc Intent Detection dataset builder class."""

  def __init__(self,
               batch_size: int,
               eval_batch_size: int,
               validation_percent: float = 0.05,
               shuffle_buffer_size: int = None,
               num_parallel_parser_calls: int = 64,
               dataset_dir: str = None):
    """Initializer.

    Args:
      batch_size: the training batch size.
      eval_batch_size: the validation and test batch size.
      validation_percent: the percent of the training set to use as a validation
        set.
      shuffle_buffer_size: the number of example to use in the shuffle buffer
        for tf.data.Dataset.shuffle().
      num_parallel_parser_calls: the number of parallel threads to use while
        preprocessing in tf.data.Dataset.map().
      dataset_dir: directory containing the TFRecord datasets and the tokenizer.
    """
    self._dataset_dir = dataset_dir
    num_validation_examples = int(_NUM_TRAIN * validation_percent)
    num_training_examples = _NUM_TRAIN - num_validation_examples


    super(JigsawToxicCommentDataset, self).__init__(
        name='jigsaw_toxic_comment',
        num_train_examples=num_training_examples,
        num_validation_examples=num_validation_examples,
        num_test_examples=_NUM_TEST,
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=shuffle_buffer_size,
        num_parallel_parser_calls=num_parallel_parser_calls)

    self.info['num_classes'] = 5  # Number of toxicity categories.
    self.info['feature_size'] = _FEATURE_LENGTH

  def _read_examples(self, split: base.Split) -> tf.data.Dataset:
    if split == base.Split.TEST:
      return _build_dataset(
          glob_dir=os.path.join(self._dataset_dir, _FILENAME_TEST),
          is_training=False)

    if split == base.Split.VAL:
      if self._num_validation_examples == 0:
        raise ValueError(
            'No validation set provided. Set `validation_percent > 0.0` to '
            'take a subset of the training set as validation.')

      # take the first _num_validation_examples of training data as validation.
      dataset = _build_dataset(
          glob_dir=os.path.join(self._dataset_dir, _FILENAME_TRAIN),
          is_training=False)
      return dataset.take(self._num_validation_examples)

    if split == base.Split.TRAIN:
      dataset = _build_dataset(
          glob_dir=os.path.join(self._dataset_dir, _FILENAME_TRAIN),
          is_training=True)
      if self._num_validation_examples > 0:
        return dataset.skip(self._num_validation_examples)

      return dataset

  def _create_process_example_fn(self, split: base.Split) -> base.PreProcessFn:

    def _example_parser(example: tf.train.Example) -> Dict[str, tf.Tensor]:
      """Parse features and labels from a serialized tf.train.Example."""
      features_spec = _make_features_spec()
      features = tf.io.parse_example(example, features_spec)
      labels = tf.cast(features.pop(_LABEL_MULTI_NAME), tf.int32)
      labels_binary = tf.cast(features.pop(_LABEL_BINARY_NAME), tf.int32)

      return {
          'features': features[_SENTENCE_ID_NAME],
          'input_mask': features[_INPUT_MASK_NAME],
          'segment_ids': features[_SEGMENT_ID_NAME],
          'labels': labels,
          'labels_binary': labels_binary,
      }

    return _example_parser



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""TF Keras definition for Resnet-20 for CIFAR."""

import functools
from typing import Any, Dict, Iterable, Optional

import tensorflow as tf


BatchNormalization = functools.partial(  # pylint: disable=invalid-name
    tf.keras.layers.BatchNormalization,
    epsilon=1e-5,  # using epsilon and momentum defaults from Torch
    momentum=0.9)
Conv2D = functools.partial(  # pylint: disable=invalid-name
    tf.keras.layers.Conv2D,
    kernel_size=3,
    padding='same',
    use_bias=False,
    kernel_initializer='he_normal')


def basic_block(
    inputs: tf.Tensor,
    filters: int,
    strides: int,
    l2: float,
    version: int) -> tf.Tensor:
  """Basic residual block of two 3x3 convs.

  Args:
    inputs: tf.Tensor.
    filters: Number of filters for Conv2D.
    strides: Stride dimensions for Conv2D.
    l2: L2 regularization coefficient.
    version: 1, indicating the original ordering from He et al. (2015); or 2,
      indicating the preactivation ordering from He et al. (2016).

  Returns:
    tf.Tensor.
  """
  x = inputs
  y = inputs
  if version == 2:
    y = BatchNormalization(beta_regularizer=tf.keras.regularizers.l2(l2),
                           gamma_regularizer=tf.keras.regularizers.l2(l2))(y)
    y = tf.keras.layers.Activation('relu')(y)
  y = Conv2D(filters,
             strides=strides,
             kernel_regularizer=tf.keras.regularizers.l2(l2))(y)
  y = BatchNormalization(beta_regularizer=tf.keras.regularizers.l2(l2),
                         gamma_regularizer=tf.keras.regularizers.l2(l2))(y)
  y = tf.keras.layers.Activation('relu')(y)
  y = Conv2D(filters,
             strides=1,
             kernel_regularizer=tf.keras.regularizers.l2(l2))(y)
  if version == 1:
    y = BatchNormalization(beta_regularizer=tf.keras.regularizers.l2(l2),
                           gamma_regularizer=tf.keras.regularizers.l2(l2))(y)
  if not x.shape.is_compatible_with(y.shape):
    x = Conv2D(filters,
               kernel_size=1,
               strides=strides,
               kernel_regularizer=tf.keras.regularizers.l2(l2))(x)
  x = tf.keras.layers.add([x, y])
  if version == 1:
    x = tf.keras.layers.Activation('relu')(x)
  return x


def group(inputs, filters, strides, num_blocks, l2, version):
  """Group of residual blocks."""
  x = basic_block(inputs, filters=filters, strides=strides, l2=l2,
                  version=version)
  for _ in range(num_blocks - 1):
    x = basic_block(x, filters=filters, strides=1, l2=l2, version=version)
  return x


def wide_resnet(
    batch_size: Optional[int],
    input_shape: Iterable[int],
    depth: int,
    width_multiplier: int,
    num_classes: int,
    l2: float,
    version: int) -> tf.keras.models.Model:
  """Builds Wide ResNet.

  Following Zagoruyko and Komodakis (2016), it accepts a width multiplier on the
  number of filters. Using three groups of residual blocks, the network maps
  spatial features of size 32x32 -> 16x16 -> 8x8.

  Args:
    batch_size: int, the batch size for the model. Can be None if the batch size
      is not statically known.
    input_shape: tf.Tensor.
    depth: Total number of convolutional layers. "n" in WRN-n-k. It differs from
      He et al. (2015)'s notation which uses the maximum depth of the network
      counting non-conv layers like dense.
    width_multiplier: Integer to multiply the number of typical filters by. "k"
      in WRN-n-k.
    num_classes: Number of output classes.
    l2: L2 regularization coefficient.
    version: 1, indicating the original ordering from He et al. (2015); or 2,
      indicating the preactivation ordering from He et al. (2016).

  Returns:
    tf.keras.Model.
  """
  if (depth - 4) % 6 != 0:
    raise ValueError('depth should be 6n+4 (e.g., 16, 22, 28, 40).')
  num_blocks = (depth - 4) // 6
  inputs = tf.keras.layers.Input(shape=input_shape, batch_size=batch_size)
  x = Conv2D(16,
             strides=1,
             kernel_regularizer=tf.keras.regularizers.l2(l2))(inputs)
  if version == 1:
    x = BatchNormalization(beta_regularizer=tf.keras.regularizers.l2(l2),
                           gamma_regularizer=tf.keras.regularizers.l2(l2))(x)
    x = tf.keras.layers.Activation('relu')(x)
  x = group(x,
            filters=16 * width_multiplier,
            strides=1,
            num_blocks=num_blocks,
            l2=l2,
            version=version)
  x = group(x,
            filters=32 * width_multiplier,
            strides=2,
            num_blocks=num_blocks,
            l2=l2,
            version=version)
  x = group(x,
            filters=64 * width_multiplier,
            strides=2,
            num_blocks=num_blocks,
            l2=l2,
            version=version)
  if version == 2:
    x = BatchNormalization(beta_regularizer=tf.keras.regularizers.l2(l2),
                           gamma_regularizer=tf.keras.regularizers.l2(l2))(x)
    x = tf.keras.layers.Activation('relu')(x)
  x = tf.keras.layers.AveragePooling2D(pool_size=8)(x)
  x = tf.keras.layers.Flatten()(x)
  x = tf.keras.layers.Dense(
      num_classes,
      kernel_initializer='he_normal',
      kernel_regularizer=tf.keras.regularizers.l2(l2),
      bias_regularizer=tf.keras.regularizers.l2(l2))(x)
  return tf.keras.Model(
      inputs=inputs,
      outputs=x,
      name='wide_resnet-{}-{}'.format(depth, width_multiplier))


def create_model(
    batch_size: Optional[int],
    depth: int,
    width_multiplier: int,
    input_shape: Iterable[int] = (32, 32, 3),
    num_classes: int = 10,
    l2_weight: float = 0.0,
    version: int = 1,
    **unused_kwargs: Dict[str, Any]) -> tf.keras.models.Model:
  return wide_resnet(
      batch_size=batch_size,
      input_shape=input_shape,
      depth=depth,
      width_multiplier=width_multiplier,
      num_classes=num_classes,
      l2=l2_weight,
      version=version)



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Utilities related to optimizers for Uncertainty Baselines."""

from typing import Any, Dict, Optional, Union
import tensorflow.compat.v2 as tf
import tensorflow_addons as tfa

LearningRateSchedule = tf.keras.optimizers.schedules.LearningRateSchedule


def get(
    optimizer_name: str,
    learning_rate: float,
    weight_decay: Optional[float] = None,
    learning_rate_schedule: Union[None, str, LearningRateSchedule] = None,
    steps_per_epoch: Optional[int] = None,
    **optimizer_kwargs: Dict[str, Any]) -> tf.keras.optimizers.Optimizer:
  """Builds a tf.keras.optimizers.Optimizer.

  Args:
    optimizer_name: the name of the optimizer to use.
    learning_rate: the base learning rate to use in a possible the learning rate
      schedule.
    weight_decay: an optional weight decay coefficient, applied using decoupled
      weight decay as per
      https://www.tensorflow.org/addons/api_docs/python/tfa/optimizers/extend_with_decoupled_weight_decay.
    learning_rate_schedule: if None, then a constant learning rate is used. Else
      a string name can be passed and a schedule created via
      get_learning_rate_schedule(). Additionally, a custom
      tf.keras.optimizers.schedules.LearningRateSchedule can be passed.
    steps_per_epoch: the number of steps per one epoch of training data.
    **optimizer_kwargs: additional kwargs passed to the optimizer.

  Returns:
    A Keras optimizer.
  """
  if isinstance(learning_rate_schedule, str):
    learning_rate = get_learning_rate_schedule(
        schedule_name=learning_rate_schedule,
        base_learning_rate=learning_rate,
        steps_per_epoch=steps_per_epoch)
  optimizer_kwargs['learning_rate'] = learning_rate

  optimizer_name = optimizer_name.lower()
  if optimizer_name == 'adam':
    optimizer_class = tf.keras.optimizers.Adam
  elif optimizer_name == 'nadam':
    optimizer_class = tf.keras.optimizers.Nadam
  elif optimizer_name == 'rmsprop':
    optimizer_class = tf.keras.optimizers.RMSprop
  elif optimizer_name in ['momentum', 'nesterov']:
    if optimizer_name == 'nesterov':
      optimizer_kwargs['nesterov'] = True
    optimizer_class = tf.keras.optimizers.SGD
  else:
    raise ValueError('Unrecognized optimizer name: {}'.format(optimizer_name))

  if weight_decay is not None and weight_decay > 0.0:
    optimizer_class = tfa.optimizers.extend_with_decoupled_weight_decay(
        optimizer_class)
    optimizer_kwargs['weight_decay'] = weight_decay

  return optimizer_class(**optimizer_kwargs)


def renset50_learning_rate_schedule(base_learning_rate, steps_per_epoch):
  """Creates a piecewise LR schedule for the common ResNet-50 ImageNet setup."""
  boundaries = [
      int(40 * steps_per_epoch),
      int(60 * steps_per_epoch),
      int(80 * steps_per_epoch),
      int(90 * steps_per_epoch),
  ]
  values = [
      base_learning_rate,
      base_learning_rate * 1e-1,
      base_learning_rate * 1e-2,
      base_learning_rate * 1e-3,
      base_learning_rate * 5e-4,
  ]
  return tf.keras.optimizers.schedules.PiecewiseConstantDecay(
      boundaries=boundaries,
      values=values)


def get_learning_rate_schedule(
    schedule_name: str,
    **schedule_kwargs: Dict[str, Any]) -> Union[float, LearningRateSchedule]:
  """Builds a step-based learning rate schedule."""
  schedule_name = schedule_name.lower()
  if schedule_name == 'constant':
    return schedule_kwargs['base_learning_rate']
  elif schedule_name == 'resnet50':
    return renset50_learning_rate_schedule(**schedule_kwargs)
  else:
    raise ValueError('Unrecognized schedule name: {}'.format(schedule_name))



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Utils for testing datasets."""

from typing import Any, Dict, Sequence, Type, TypeVar

import tensorflow.compat.v2 as tf
from uncertainty_baselines.datasets import base


class DatasetTest(tf.test.TestCase):
  """Utility class for testing dataset construction."""

  def _testDatasetSize(
      self,
      dataset_class: Type[TypeVar('B', bound=base.BaseDataset)],
      image_size: Sequence[int],
      **kwargs: Dict[str, Any]):
    batch_size = 9
    eval_batch_size = 5
    dataset_builder = dataset_class(
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=20,
        **kwargs)
    for split in base.Split:
      dataset = dataset_builder.build(split).take(1)
      element = next(iter(dataset))
      features = element['features']
      labels = element['labels']

      features_shape = features.shape
      labels_shape = labels.shape
      if split == base.Split.TRAIN:
        expected_bs = batch_size
      else:
        expected_bs = eval_batch_size
      self.assertEqual(
          features_shape, tuple([expected_bs] + list(image_size)))
      self.assertEqual(labels_shape, (expected_bs,))


if __name__ == '__main__':
  tf.test.main()



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Uncertainty baseline training."""

import importlib
from absl import logging


_IMPORTS = [
    'datasets', 'models', 'optimizers', 'strategy_utils', 'utils'
]


def _lazy_import(name):
  module = importlib.import_module(__name__)
  imported = importlib.import_module('.' + name, 'uncertainty_baselines')
  setattr(module, name, imported)
  return imported


for module_name in _IMPORTS:
  try:
    _lazy_import(module_name)
  except ModuleNotFoundError as e:
    logging.warning(e)




# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Collection of shared utility functions."""

from typing import Any, Callable, Dict
from absl import logging
import numpy as np

import tensorflow.compat.v2 as tf
from uncertainty_baselines.datasets import base


def assert_weights_loaded(model: tf.keras.Model, before_restore_vars):
  """Assert that all variables changed after restoring."""
  after_restore_vars = model.trainable_weights
  for wi, (before_var, after_var) in enumerate(
      zip(before_restore_vars, after_restore_vars)):
    weight_i = model.trainable_weights[wi]
    logging.warning(
        'Weight %s norm before restore %f and after %f.',
        weight_i.name,
        np.linalg.norm(before_var),
        np.linalg.norm(after_var))
    if ('kernel/linear_bias' not in weight_i.name and
        np.array_equal(before_var, after_var)):
      error_message = (
          'Weight #{} ({}) was not restored properly. Norm at '
          'initialization: {}, norm after restoration: {}').format(
              wi,
              weight_i,
              np.linalg.norm(before_var),
              np.linalg.norm(after_var))
      raise ValueError(error_message)


def build_dataset(
    dataset_builder: base.BaseDataset,
    strategy,
    split: str,
    as_tuple: bool = False) -> tf.data.Dataset:
  """Build a tf.data.Dataset iterator using a DistributionStrategy."""
  dataset = dataset_builder.build(split, as_tuple=as_tuple)
  # TOOD(znado): look into using experimental_distribute_datasets_from_function
  # and a wrapped version of dataset_builder.build (see
  # https://www.tensorflow.org/api_docs/python/tf/distribute/Strategy#experimental_distribute_datasets_from_function).
  # Can experimental_distribute_dataset properly shard TFDS datasets?
  if strategy.num_replicas_in_sync > 1:
    dataset = strategy.experimental_distribute_dataset(dataset)
  return dataset


_TensorDict = Dict[str, tf.Tensor]
_StepFn = Callable[[_TensorDict], _TensorDict]


def call_step_fn(
    strategy,
    step_fn: _StepFn,
    global_inputs: Any) -> Dict[str, float]:
  """Call the step_fn on the iterator output using DistributionStrategy."""
  step_outputs = strategy.run(step_fn, args=(global_inputs,))
  if not step_outputs:
    return step_outputs
  if strategy.num_replicas_in_sync > 1:
    step_outputs = {
        name: strategy.reduce(tf.distribute.ReduceOp.SUM, (output,), axis=0)
        for name, output in step_outputs.items()
    }
    step_outputs = {
        k: v / strategy.num_replicas_in_sync
        for k, v in step_outputs.items()
    }
  return step_outputs


# TODO(znado): remove when uncertainty metrics has this implemented.
def compute_accuracy(labels: tf.Tensor, logits: tf.Tensor) -> tf.Tensor:
  """Computes classification accuracy given logits and dense labels.

  Args:
    labels: Integer Tensor of dense labels, shape [batch_size].
    logits: Tensor of shape [batch_size, num_classes].
  Returns:
    A scalar for the classification accuracy.
  """
  correct_prediction = tf.equal(
      tf.argmax(logits, 1, output_type=tf.int32), labels)
  return tf.reduce_mean(tf.cast(correct_prediction, tf.float32))



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3

"""Provides utilities to preprocess images for the Inception networks."""

from typing import Callable, Tuple, Optional
import tensorflow.compat.v1 as tf


# TODO(znado): update this to use TF types (TensorLike) when that module is
# ready.
def apply_with_random_selector(
    x: tf.Tensor,
    func: Callable[[tf.Tensor, tf.Tensor], tf.Tensor],
    num_cases: int,
    selected: Optional[int] = None) -> tf.Tensor:
  """Computes func(x, sel), with sel sampled from [0...num_cases-1].

  Args:
    x: input Tensor.
    func: Python function to apply.
    num_cases: Python int32, number of cases to sample sel from.
    selected: Python int32, optional value to use as the selected index.

  Returns:
    The result of func(x, sel), where func receives the value of the
    selector as a python integer, but sel is sampled dynamically.
  """
  if selected is None:
    selected = tf.random.uniform([], maxval=num_cases, dtype=tf.int32)
  branches = [
      lambda i=case: func(x, i)
      for case in range(num_cases)
  ]
  return tf.switch_case(selected, branches)


def distort_color(
    image: tf.Tensor,
    color_ordering: int = 0,
    fast_mode: bool = True,
    scope: bool = None) -> tf.Tensor:
  """Distort the color of a Tensor image.

  Each color distortion is non-commutative and thus ordering of the color ops
  matters. Ideally we would randomly permute the ordering of the color ops.
  Rather then adding that level of complication, we select a distinct ordering
  of color ops for each preprocessing thread.

  Args:
    image: 3-D Tensor containing single image in [0, 1].
    color_ordering: Python int, a type of distortion (valid values: 0-3).
    fast_mode: Avoids slower ops (random_hue and random_contrast)
    scope: Optional scope for name_scope.
  Returns:
    3-D Tensor color-distorted image on range [0, 1]
  Raises:
    ValueError: if color_ordering not in [0, 3]
  """
  with tf.name_scope(scope, 'distort_color', [image]):
    if fast_mode:
      if color_ordering == 0:
        image = tf.image.random_brightness(image, max_delta=32. / 255.)
        image = tf.image.random_saturation(image, lower=0.5, upper=1.5)
      else:
        image = tf.image.random_saturation(image, lower=0.5, upper=1.5)
        image = tf.image.random_brightness(image, max_delta=32. / 255.)
    else:
      if color_ordering == 0:
        image = tf.image.random_brightness(image, max_delta=32. / 255.)
        image = tf.image.random_saturation(image, lower=0.5, upper=1.5)
        image = tf.image.random_hue(image, max_delta=0.2)
        image = tf.image.random_contrast(image, lower=0.5, upper=1.5)
      elif color_ordering == 1:
        image = tf.image.random_saturation(image, lower=0.5, upper=1.5)
        image = tf.image.random_brightness(image, max_delta=32. / 255.)
        image = tf.image.random_contrast(image, lower=0.5, upper=1.5)
        image = tf.image.random_hue(image, max_delta=0.2)
      elif color_ordering == 2:
        image = tf.image.random_contrast(image, lower=0.5, upper=1.5)
        image = tf.image.random_hue(image, max_delta=0.2)
        image = tf.image.random_brightness(image, max_delta=32. / 255.)
        image = tf.image.random_saturation(image, lower=0.5, upper=1.5)
      elif color_ordering == 3:
        image = tf.image.random_hue(image, max_delta=0.2)
        image = tf.image.random_saturation(image, lower=0.5, upper=1.5)
        image = tf.image.random_contrast(image, lower=0.5, upper=1.5)
        image = tf.image.random_brightness(image, max_delta=32. / 255.)
      else:
        raise ValueError('color_ordering must be in [0, 3]')

    # The random_* ops do not necessarily clamp.
    return tf.clip_by_value(image, 0.0, 1.0)


def distorted_bounding_box_crop(
    image: tf.Tensor,
    bbox: tf.Tensor,
    min_object_covered: float = 0.1,
    aspect_ratio_range: Tuple[float, float] = (0.75, 1.33),
    area_range: Tuple[float, float] = (0.05, 1.0),
    max_attempts: int = 100,
    scope: Optional[str] = None) -> Tuple[tf.Tensor, tf.Tensor]:
  """Generates cropped_image using a one of the bboxes randomly distorted.

  See `tf.image.sample_distorted_bounding_box` for more documentation.

  Args:
    image: 3-D Tensor of image (it will be converted to floats in [0, 1]).
    bbox: 3-D float Tensor of bounding boxes arranged [1, num_boxes, coords]
      where each coordinate is [0, 1) and the coordinates are arranged
      as [ymin, xmin, ymax, xmax]. If num_boxes is 0 then it would use the whole
      image.
    min_object_covered: An optional `float`. Defaults to `0.1`. The cropped
      area of the image must contain at least this fraction of any bounding box
      supplied.
    aspect_ratio_range: An optional list of `floats`. The cropped area of the
      image must have an aspect ratio = width / height within this range.
    area_range: An optional list of `floats`. The cropped area of the image
      must contain a fraction of the supplied image within in this range.
    max_attempts: An optional `int`. Number of attempts at generating a cropped
      region of the image of the specified constraints. After `max_attempts`
      failures, return the entire image.
    scope: Optional scope for name_scope.
  Returns:
    A tuple, a 3-D Tensor cropped_image and the distorted bbox
  """
  with tf.name_scope(scope, 'distorted_bounding_box_crop', [image, bbox]):
    # Each bounding box has shape [1, num_boxes, box coords] and
    # the coordinates are ordered [ymin, xmin, ymax, xmax].

    # A large fraction of image datasets contain a human-annotated bounding
    # box delineating the region of the image containing the object of interest.
    # We choose to create a new bounding box for the object which is a randomly
    # distorted version of the human-annotated bounding box that obeys an
    # allowed range of aspect ratios, sizes and overlap with the human-annotated
    # bounding box. If no box is supplied, then we assume the bounding box is
    # the entire image.
    sample_distorted_bounding_box = tf.image.sample_distorted_bounding_box(
        tf.shape(image),
        bounding_boxes=bbox,
        min_object_covered=min_object_covered,
        aspect_ratio_range=aspect_ratio_range,
        area_range=area_range,
        max_attempts=max_attempts,
        use_image_if_no_bounding_boxes=True)
    bbox_begin, bbox_size, distort_bbox = sample_distorted_bounding_box

    # Crop the image to the specified bounding box.
    cropped_image = tf.slice(image, bbox_begin, bbox_size)
    return cropped_image, distort_bbox


def preprocess_for_train(
    image: tf.Tensor,
    height: int,
    width: int,
    bbox: tf.Tensor,
    fast_mode: bool = True,
    scope: Optional[str] = None,
    add_image_summaries: bool = True,
    random_crop: bool = True,
    use_grayscale: bool = False) -> tf.Tensor:
  """Distort one image for training a network.

  Distorting images provides a useful technique for augmenting the data
  set during training in order to make the network invariant to aspects
  of the image that do not effect the label.

  Additionally it would create image_summaries to display the different
  transformations applied to the image.

  Args:
    image: 3-D Tensor of image. If dtype is tf.float32 then the range should be
      [0, 1], otherwise it would converted to tf.float32 assuming that the range
      is [0, MAX], where MAX is largest positive representable number for
      int(8/16/32) data type (see `tf.image.convert_image_dtype` for details).
    height: integer
    width: integer
    bbox: 3-D float Tensor of bounding boxes arranged [1, num_boxes, coords]
      where each coordinate is [0, 1) and the coordinates are arranged
      as [ymin, xmin, ymax, xmax].
    fast_mode: Optional boolean, if True avoids slower transformations (i.e.
      bi-cubic resizing, random_hue or random_contrast).
    scope: Optional scope for name_scope.
    add_image_summaries: Enable image summaries.
    random_crop: Enable random cropping of images during preprocessing for
      training.
    use_grayscale: Whether to convert the image from RGB to grayscale.
  Returns:
    3-D float Tensor of distorted image used for training with range [-1, 1].
  """
  with tf.name_scope(scope, 'distort_image', [image, height, width, bbox]):
    if bbox is None:
      bbox = tf.constant([0.0, 0.0, 1.0, 1.0],
                         dtype=tf.float32,
                         shape=[1, 1, 4])
    if image.dtype != tf.float32:
      image = tf.image.convert_image_dtype(image, dtype=tf.float32)
    # Each bounding box has shape [1, num_boxes, box coords] and
    # the coordinates are ordered [ymin, xmin, ymax, xmax].
    image_with_box = tf.image.draw_bounding_boxes(tf.expand_dims(image, 0),
                                                  bbox)
    if add_image_summaries:
      tf.summary.image('image_with_bounding_boxes', image_with_box)

    if not random_crop:
      distorted_image = image
    else:
      distorted_image, distorted_bbox = distorted_bounding_box_crop(image, bbox)
      # Restore the shape since the dynamic slice based upon the bbox_size loses
      # the third dimension.
      distorted_image.set_shape([None, None, 3])
      image_with_distorted_box = tf.image.draw_bounding_boxes(
          tf.expand_dims(image, 0), distorted_bbox)
      if add_image_summaries:
        tf.summary.image('images_with_distorted_bounding_box',
                         image_with_distorted_box)

    # This resizing operation may distort the images because the aspect
    # ratio is not respected. We select a resize method in a round robin
    # fashion based on the thread number.
    # Note that ResizeMethod contains 4 enumerated resizing methods.

    methods = ['bilinear', 'nearest', 'bicubic', 'area']
    # We select only 1 case for fast_mode bilinear.
    num_resize_cases = 1 if fast_mode else 4
    distorted_image = apply_with_random_selector(
        distorted_image,
        lambda x, method: tf.image.resize(x, [height, width], methods[method]),
        num_cases=num_resize_cases)

    if add_image_summaries:
      tf.summary.image(('cropped_' if random_crop else '') + 'resized_image',
                       tf.expand_dims(distorted_image, 0))

    # Randomly flip the image horizontally.
    distorted_image = tf.image.random_flip_left_right(distorted_image)

    # Randomly distort the colors. There are 1 or 4 ways to do it.
    num_distort_cases = 1 if fast_mode else 4
    distorted_image = apply_with_random_selector(
        distorted_image,
        lambda x, ordering: distort_color(x, ordering, fast_mode),
        num_cases=num_distort_cases)

    if use_grayscale:
      distorted_image = tf.image.rgb_to_grayscale(distorted_image)

    if add_image_summaries:
      tf.summary.image('final_distorted_image',
                       tf.expand_dims(distorted_image, 0))
    distorted_image = tf.subtract(distorted_image, 0.5)
    distorted_image = tf.multiply(distorted_image, 2.0)
    return distorted_image


def preprocess_for_eval(
    image: tf.Tensor,
    height: int,
    width: int,
    central_fraction: float = 0.875,
    scope: Optional[str] = None,
    central_crop: bool = True,
    use_grayscale: bool = False) -> tf.Tensor:
  """Prepare one image for evaluation.

  If height and width are specified it would output an image with that size by
  applying resize_bilinear.

  If central_fraction is specified it would crop the central fraction of the
  input image.

  Args:
    image: 3-D Tensor of image. If dtype is tf.float32 then the range should be
      [0, 1], otherwise it would converted to tf.float32 assuming that the range
      is [0, MAX], where MAX is largest positive representable number for
      int(8/16/32) data type (see `tf.image.convert_image_dtype` for details).
    height: integer
    width: integer
    central_fraction: Optional Float, fraction of the image to crop.
    scope: Optional scope for name_scope.
    central_crop: Enable central cropping of images during preprocessing for
      evaluation.
    use_grayscale: Whether to convert the image from RGB to grayscale.
  Returns:
    3-D float Tensor of prepared image.
  """
  with tf.name_scope(scope, 'eval_image', [image, height, width]):
    if image.dtype != tf.float32:
      image = tf.image.convert_image_dtype(image, dtype=tf.float32)
    if use_grayscale:
      image = tf.image.rgb_to_grayscale(image)
    # Crop the central region of the image with an area containing 87.5% of
    # the original image.
    if central_crop and central_fraction:
      image = tf.image.central_crop(image, central_fraction=central_fraction)

    if height and width:
      # Resize the image to the specified height and width.
      image = tf.expand_dims(image, 0)
      image = tf.image.resize_bilinear(image, [height, width],
                                       align_corners=False)
      image = tf.squeeze(image, [0])
    image = tf.subtract(image, 0.5)
    image = tf.multiply(image, 2.0)
    return image


def preprocess_image(
    image: tf.Tensor,
    height: int,
    width: int,
    is_training: bool = False,
    bbox: Optional[tf.Tensor] = None,
    fast_mode: bool = True,
    add_image_summaries: bool = True,
    crop_image: bool = True,
    use_grayscale: bool = False) -> tf.Tensor:
  """Pre-process one image for training or evaluation.

  Args:
    image: 3-D Tensor [height, width, channels] with the image. If dtype is
      tf.float32 then the range should be [0, 1], otherwise it would converted
      to tf.float32 assuming that the range is [0, MAX], where MAX is largest
      positive representable number for int(8/16/32) data type (see
      `tf.image.convert_image_dtype` for details).
    height: integer, image expected height.
    width: integer, image expected width.
    is_training: Boolean. If true it would transform an image for train,
      otherwise it would transform it for evaluation.
    bbox: 3-D float Tensor of bounding boxes arranged [1, num_boxes, coords]
      where each coordinate is [0, 1) and the coordinates are arranged as
      [ymin, xmin, ymax, xmax].
    fast_mode: Optional boolean, if True avoids slower transformations.
    add_image_summaries: Enable image summaries.
    crop_image: Whether to enable cropping of images during preprocessing for
      both training and evaluation.
    use_grayscale: Whether to convert the image from RGB to grayscale.

  Returns:
    3-D float Tensor containing an appropriately scaled image

  Raises:
    ValueError: if user does not provide bounding box
  """
  if is_training:
    return preprocess_for_train(
        image,
        height,
        width,
        bbox,
        fast_mode,
        add_image_summaries=add_image_summaries,
        random_crop=crop_image,
        use_grayscale=use_grayscale)
  else:
    return preprocess_for_eval(
        image,
        height,
        width,
        central_crop=crop_image,
        use_grayscale=use_grayscale)



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Abstract base classes which defines interfaces for datasets."""

import abc
import enum
from typing import Callable, Dict, Optional, Sequence, Union
from absl import logging
import six
import tensorflow.compat.v2 as tf


class Split(enum.Enum):
  TRAIN = 'train'
  VAL = 'validation'
  TEST = 'test'


# For datasets like UCI, the tf.data.Dataset returned by _read_examples will
# have elements that are Sequence[tf.Tensor], but for TFDS datasets they will be
# Dict[str, tf.Tensor].
PreProcessFn = Callable[
    [Union[Sequence[tf.Tensor], Dict[str, tf.Tensor]]], Dict[str, tf.Tensor]]


@six.add_metaclass(abc.ABCMeta)
class BaseDataset(object):
  """Abstract base dataset class.

  Requires subclasses to override _read_examples, _create_process_example_fn.
  """

  def __init__(
      self,
      name: str,
      batch_size: int,
      eval_batch_size: int,
      num_train_examples: int,
      num_validation_examples: int,
      num_test_examples: int,
      shuffle_buffer_size: int = None,
      num_parallel_parser_calls: int = 64):
    """Create a tf.data.Dataset builder.

    Args:
      name: the name of this dataset.
      batch_size: the training batch size.
      eval_batch_size: the validation and test batch size.
      num_train_examples: the number of training examples in this dataset.
      num_validation_examples: the number of validation examples in this
        dataset.
      num_test_examples: the number of test examples in this dataset.
      shuffle_buffer_size: the number of example to use in the shuffle buffer
        for tf.data.Dataset.shuffle().
      num_parallel_parser_calls: the number of parallel threads to use while
        preprocessing in tf.data.Dataset.map().
    """
    self.name = name

    self.batch_size = batch_size
    self.eval_batch_size = eval_batch_size
    self.info = {
        'num_train_examples': num_train_examples,
        'num_validation_examples': num_validation_examples,
        'num_test_examples': num_test_examples,
    }

    if shuffle_buffer_size is None:
      self._shuffle_buffer_size = num_train_examples
    else:
      self._shuffle_buffer_size = shuffle_buffer_size

    self._num_parallel_parser_calls = num_parallel_parser_calls

  def _is_training(self, split: Split) -> bool:
    return split == Split.TRAIN

  @property
  def _num_train_examples(self) -> int:
    return self.info['num_train_examples']

  @property
  def _num_validation_examples(self) -> int:
    return self.info['num_validation_examples']

  @property
  def _num_test_examples(self) -> int:
    return self.info['num_test_examples']

  def _read_examples(self, split: Split) -> tf.data.Dataset:
    """Return a tf.data.Dataset to be used by _create_process_example_fn."""
    raise NotImplementedError('Must override dataset _read_examples!')

  def _create_process_example_fn(self, split: Split) -> Optional[PreProcessFn]:
    """Create a function to perform dataset pre-processing, if needed.

    Args:
      split: A dataset split.

    Returns:
      None if no processing is necessary. Otherwise, a function which takes as
      inputs a single element of the dataset (passed from dataset.map()), and
      returns a dict with keys 'features' and 'labels' and their corresponding
      Tensor values.
    """
    raise NotImplementedError(
        'Must override dataset _create_process_example_fn!')

  def _batch(self, split: Split, dataset: tf.data.Dataset) -> tf.data.Dataset:
    """Get the batched version of `dataset`."""
    # `uneven_datasets` is a list of datasets with a number of validation and/or
    # test examples that is not evenly divisible by commonly used batch sizes.
    uneven_datasets = ['criteo', 'svhn']
    if self._is_training(split):
      batch_size = self.batch_size
    elif split == Split.VAL:
      batch_size = self.eval_batch_size
      if (self._num_validation_examples % batch_size != 0 and
          self.name not in uneven_datasets):
        logging.warn(
            'Batch size does not evenly divide the number of validation '
            'examples , cannot ensure static shapes on TPU. Batch size: %d, '
            'validation examples: %d',
            batch_size,
            self._num_validation_examples)
    else:
      batch_size = self.eval_batch_size
      if (self._num_test_examples % batch_size != 0 and
          self.name not in uneven_datasets):
        logging.warn(
            'Batch size does not evenly divide the number of test examples, '
            'cannot ensure static shapes on TPU. Batch size: %d, test '
            'examples: %d', batch_size, self._num_test_examples)
    # Note that we always drop the last batch when the batch size does not
    # evenly divide the number of examples.
    return dataset.batch(batch_size, drop_remainder=True)

  def build(
      self,
      split: Union[str, Split],
      as_tuple: bool = False) -> tf.data.Dataset:
    """Transforms the dataset from self._read_examples() to batch, repeat, etc.

    Note that we do not handle replication/sharding here, because the
    DistributionStrategy experimental_distribute_dataset() will shard the input
    files for us.

    Args:
      split: a dataset split, either one of the Split enum or their associated
        strings.
      as_tuple: whether or not to return a Dataset where each element is a Dict
        with at least the keys ['features', 'labels'], or a tuple of
        (feature, label). If there are keys besides 'features' and 'labels' in
        the Dict then this ignore them.

    Returns:
      A tf.data.Dataset of elements that are a dict with keys 'features' and
      'labels' and their corresponding Tensor values.
    """
    if isinstance(split, str):
      split = Split(split)

    dataset = self._read_examples(split)
    process_example_fn = self._create_process_example_fn(split)
    # Map the parser over the dataset.
    if process_example_fn:
      dataset = dataset.map(
          process_example_fn,
          num_parallel_calls=self._num_parallel_parser_calls)
    if as_tuple:
      dataset = dataset.map(lambda d: (d['features'], d['labels']))
    dataset = dataset.cache()

    # Shuffle and repeat only for the training split.
    if self._is_training(split):
      dataset = dataset.shuffle(self._shuffle_buffer_size)
      dataset = dataset.repeat()

    dataset = self._batch(split, dataset)

    dataset = dataset.prefetch(-1)

    options = tf.data.Options()
    options.experimental_distribute.auto_shard_policy = (
        tf.data.experimental.AutoShardPolicy.OFF)
    dataset = dataset.with_options(options)

    return dataset



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""CIFAR{10,100} dataset builders."""

from typing import Any, Dict

import tensorflow.compat.v2 as tf
import tensorflow_datasets as tfds
from uncertainty_baselines.datasets import base


class _CifarDataset(base.BaseDataset):
  """CIFAR dataset builder abstract class."""

  def __init__(
      self,
      name: str,
      batch_size: int,
      eval_batch_size: int,
      validation_percent: float = 0.0,
      shuffle_buffer_size: int = None,
      num_parallel_parser_calls: int = 64,
      normalize: bool = True,
      **unused_kwargs: Dict[str, Any]):
    """Create a CIFAR10 or CIFAR100 tf.data.Dataset builder.

    Args:
      name: the name of this dataset, either 'cifar10' or 'cifar100'.
      batch_size: the training batch size.
      eval_batch_size: the validation and test batch size.
      validation_percent: the percent of the training set to use as a validation
        set.
      shuffle_buffer_size: the number of example to use in the shuffle buffer
        for tf.data.Dataset.shuffle().
      num_parallel_parser_calls: the number of parallel threads to use while
        preprocessing in tf.data.Dataset.map().
      normalize: whether or not to normalize each image by the CIFAR dataset
        mean and stddev.
    """
    self._normalize = normalize
    num_train_examples = 50000
    num_validation_examples = int(num_train_examples * validation_percent)
    num_train_examples -= num_validation_examples
    super(_CifarDataset, self).__init__(
        name=name,
        num_train_examples=num_train_examples,
        num_validation_examples=num_validation_examples,
        num_test_examples=10000,
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=shuffle_buffer_size,
        num_parallel_parser_calls=num_parallel_parser_calls)

  def _read_examples(self, split: base.Split) -> tf.data.Dataset:
    """Creates a dataset to be processed by _create_process_example_fn."""
    if split == base.Split.TEST:
      return tfds.load(self.name + ':3.*.*', split='test')

    if split == base.Split.TRAIN:
      if self._num_validation_examples == 0:
        train_split = 'train'
      else:
        train_split = tfds.core.ReadInstruction(
            'train', to=-self._num_validation_examples, unit='abs')
      return tfds.load(self.name + ':3.*.*', split=train_split)
    if split == base.Split.VAL:
      if self._num_validation_examples == 0:
        raise ValueError(
            'No validation set provided. Set `validation_percent > 0.0` to '
            'take a subset of the training set as validation.')
      val_split = tfds.core.ReadInstruction(
          'train', from_=-self._num_validation_examples, unit='abs')
      return tfds.load(self.name + ':3.*.*', split=val_split)

  def _create_process_example_fn(self, split: base.Split) -> base.PreProcessFn:

    def _example_parser(example: Dict[str, tf.Tensor]) -> Dict[str, tf.Tensor]:
      """A pre-process function to return images in [0, 1]."""
      image = example['image']
      label = example['label']
      if self._is_training(split):
        # Expand the image by 2 pixels, then crop back down to 32x32.
        image = tf.image.resize_with_crop_or_pad(image, 32 + 4, 32 + 4)
        image = tf.image.random_crop(image, [32, 32, 3])
        image = tf.image.random_flip_left_right(image)

      # Normalize the values of the image to be in [0, 1].
      if image.dtype != tf.uint8:
        raise ValueError(
            'Images need to be type uint8 to use tf.image.convert_image_dtype.')
      # The image has values in the range [0, 1].
      image = tf.image.convert_image_dtype(image, tf.float32)
      # Optionally normalize by the dataset statistics.
      if self._normalize:
        mean = tf.constant([0.4914, 0.4822, 0.4465])
        std = tf.constant([0.2023, 0.1994, 0.2010])
        image = (image - mean) / std
      return {'features': image, 'labels': tf.cast(label, tf.int32)}

    return _example_parser


class Cifar10Dataset(_CifarDataset):
  """CIFAR10 dataset builder class."""

  def __init__(self, **kwargs):
    super(Cifar10Dataset, self).__init__(name='cifar10', **kwargs)


class Cifar100Dataset(_CifarDataset):
  """CIFAR100 dataset builder class."""

  def __init__(self, **kwargs):
    super(Cifar100Dataset, self).__init__(name='cifar100', **kwargs)



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for uncertainty_baselines.datasets.mnli."""

from absl.testing import parameterized

import tensorflow as tf

from uncertainty_baselines.datasets import base
from uncertainty_baselines.datasets import mnli


class MnliTest(tf.test.TestCase, parameterized.TestCase):

  @parameterized.named_parameters(('ind,train', 'matched', base.Split.TRAIN),
                                  ('ind,valid', 'matched', base.Split.VAL),
                                  ('ind,test', 'matched', base.Split.TEST),
                                  ('ood,valid', 'mismatched', base.Split.VAL),
                                  ('ood,test', 'mismatched', base.Split.TEST))
  def testDatasetSize(self, mode, split):
    batch_size = 9
    eval_batch_size = 5
    dataset_builder = mnli.MnliDataset(
        mode=mode,
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=20)
    dataset = dataset_builder.build(split).take(1)
    element = next(iter(dataset))
    text_a = element['text_a']
    text_b = element['text_b']
    labels = element['labels']

    expected_batch_size = (
        batch_size if split == base.Split.TRAIN else eval_batch_size)
    feature_a_shape = text_a.shape[0]
    feature_b_shape = text_b.shape[0]
    labels_shape = labels.shape[0]

    self.assertEqual(feature_a_shape, expected_batch_size)
    self.assertEqual(feature_b_shape, expected_batch_size)
    self.assertEqual(labels_shape, expected_batch_size)

  def testTrainSplitError(self):
    """Tests for ValueError when calling train split for mismatched data."""
    batch_size = 9
    eval_batch_size = 5
    dataset_builder = mnli.MnliDataset(
        mode='mismatched',
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=20)

    with self.assertRaises(Exception):
      dataset_builder.build(base.Split.TRAIN)


if __name__ == '__main__':
  tf.test.main()



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Data loader for the Criteo dataset."""

import os.path
from typing import Any, Dict

import tensorflow.compat.v2 as tf
from uncertainty_baselines.datasets import base


NUM_INT_FEATURES = 13  # Number of Criteo integer features.
NUM_CAT_FEATURES = 26  # Number of Criteo categorical features.
NUM_TOTAL_FEATURES = NUM_INT_FEATURES + NUM_CAT_FEATURES

_INT_KEY_TMPL = 'int-feature-%d'
_CAT_KEY_TMPL = 'categorical-feature-%d'


def _build_dataset(glob_dir: str, is_training: bool) -> tf.data.Dataset:
  cycle_len = 10 if is_training else 1
  dataset = tf.data.Dataset.list_files(glob_dir, shuffle=is_training)
  dataset = dataset.interleave(tf.data.TFRecordDataset, cycle_length=cycle_len)
  return dataset


def feature_name(idx: int) -> str:
  assert 0 < idx <= NUM_TOTAL_FEATURES
  if idx <= NUM_INT_FEATURES:
    return _INT_KEY_TMPL % idx
  return _CAT_KEY_TMPL % idx


def _make_features_spec() -> Dict[str, tf.io.FixedLenFeature]:
  features = {'clicked': tf.io.FixedLenFeature([1], tf.float32)}
  for idx in range(1, NUM_INT_FEATURES + 1):
    features[feature_name(idx)] = tf.io.FixedLenFeature([1], tf.float32, -1)
  for idx in range(NUM_INT_FEATURES + 1, NUM_TOTAL_FEATURES + 1):
    features[feature_name(idx)] = tf.io.FixedLenFeature([1], tf.string, '')
  return features


class CriteoDataset(base.BaseDataset):
  """Criteo dataset builder class."""

  def __init__(
      self,
      batch_size: int,
      eval_batch_size: int,
      shuffle_buffer_size: int = None,
      num_parallel_parser_calls: int = 64,
      dataset_dir: str = None,
      **unused_kwargs: Dict[str, Any]):
    """Create a Criteo tf.data.Dataset builder.

    Args:
      batch_size: the training batch size.
      eval_batch_size: the validation and test batch size.
      shuffle_buffer_size: the number of example to use in the shuffle buffer
        for tf.data.Dataset.shuffle().
      num_parallel_parser_calls: the number of parallel threads to use while
        preprocessing in tf.data.Dataset.map().
      dataset_dir: path to a directory containing the Criteo datasets, with
        filenames train-*-of-*', 'validate.tfr', 'test.tfr'.
    """
    self._dataset_dir = dataset_dir
    super(CriteoDataset, self).__init__(
        name='criteo',
        num_train_examples=int(37e6),
        num_validation_examples=4420308,
        num_test_examples=4420309,
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=shuffle_buffer_size,
        num_parallel_parser_calls=num_parallel_parser_calls)

  def _read_examples(self, split: base.Split) -> tf.data.Dataset:
    if split == base.Split.TRAIN:
      return _build_dataset(
          glob_dir=os.path.join(self._dataset_dir, 'train-*-of-*'),
          is_training=True)
    if split == base.Split.VAL:
      return _build_dataset(
          glob_dir=os.path.join(self._dataset_dir, 'validation-*-of-*'),
          is_training=False)
    return _build_dataset(
        glob_dir=os.path.join(self._dataset_dir, 'test-*-of-*'),
        is_training=False)

  def _create_process_example_fn(self, split: base.Split) -> base.PreProcessFn:

    def _example_parser(example: tf.train.Example) -> Dict[str, tf.Tensor]:
      """Parse features and labels from a serialized tf.train.Example."""
      features_spec = _make_features_spec()
      features = tf.io.parse_example(example, features_spec)
      features = {k: tf.squeeze(v, axis=0) for k, v in features.items()}
      labels = tf.cast(features.pop('clicked'), tf.int32)
      return {'features': features, 'labels': labels}

    return _example_parser



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Uncertainty baseline training models."""

# When adding a new model, also add to models.py for easier user access.
from uncertainty_baselines.models.bert import create_model as BERTBuilder
from uncertainty_baselines.models.criteo_mlp import create_model as CriteoMlpBuilder
from uncertainty_baselines.models.models import get
from uncertainty_baselines.models.resnet20 import create_model as ResNet20Builder
from uncertainty_baselines.models.resnet50 import create_model as ResNet50Builder
from uncertainty_baselines.models.textcnn import create_model as TextCNNBuilder
from uncertainty_baselines.models.wide_resnet import create_model as WideResNetBuilder



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Tests for ClincIntentDetectionDataset."""

from absl.testing import parameterized
import numpy as np
import tensorflow.compat.v2 as tf
import uncertainty_baselines as ub
from uncertainty_baselines.datasets import base
from uncertainty_baselines.datasets import clinc_intent


class ClincIntentDetectionDatasetTest(tf.test.TestCase, parameterized.TestCase):

  @parameterized.named_parameters(('Train', base.Split.TRAIN),
                                  ('Validation', base.Split.VAL),
                                  ('Test', base.Split.TEST))
  def testDatasetSize(self, split):
    batch_size = 9
    eval_batch_size = 5
    dataset_builder = ub.datasets.ClincIntentDetectionDataset(
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=20)
    dataset = dataset_builder.build(split).take(1)
    element = next(iter(dataset))
    features = element['features']
    labels = element['labels']

    expected_batch_size = (
        batch_size if split == base.Split.TRAIN else eval_batch_size)
    feature_shape, _ = features.shape
    labels_shape = labels.shape
    self.assertEqual(feature_shape, expected_batch_size)
    self.assertEqual(labels_shape, (expected_batch_size,))

  @parameterized.named_parameters(('IND', 'ind', clinc_intent._NUM_TRAIN_IND),
                                  ('OOD', 'ood', clinc_intent._NUM_TRAIN_OOD),
                                  ('All', 'all', clinc_intent._NUM_TRAIN_ALL))
  def testDataMode(self, data_mode, num_train_examples_expected):
    """Tests if all data modes can be loaded correctly."""
    batch_size = 9
    eval_batch_size = 5
    split = base.Split.TRAIN

    dataset_builder = ub.datasets.ClincIntentDetectionDataset(
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=20,
        data_mode=data_mode)

    num_train_examples = dataset_builder.info['num_train_examples']
    dataset = dataset_builder.build(split).take(1)
    element = next(iter(dataset))
    features = element['features']

    _, features_length = features.shape

    self.assertEqual(features_length, clinc_intent._FEATURE_LENGTH)
    self.assertEqual(num_train_examples, num_train_examples_expected)

  def testTokenizer(self):
    """Tests if tokenizer is loaded correctly."""
    dataset_builder = ub.datasets.ClincIntentDetectionDataset(
        batch_size=9,
        eval_batch_size=5,
        shuffle_buffer_size=20,
    )

    # The number of valid tokens.
    vocab_size = dataset_builder.tokenizer.num_words

    self.assertEqual(vocab_size, 7291)

  def testNumTokens(self):
    """Tests if num_tokens field is loaded correctly."""
    batch_size = 9
    eval_batch_size = 5
    split = base.Split.TRAIN

    dataset_builder = ub.datasets.ClincIntentDetectionDataset(
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=20,)

    dataset = dataset_builder.build(split).take(1)
    element = next(iter(dataset))
    features = element['features']
    num_tokens = element['num_tokens']

    # compute number of tokens expected
    num_tokens_expected = np.sum(features.numpy() != 0, axis=-1)
    num_tokens_loaded = num_tokens.numpy()

    np.testing.assert_array_equal(num_tokens_loaded, num_tokens_expected)

if __name__ == '__main__':
  tf.test.main()



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Data loader for the General Language Understanding Evaluation (GLUE) benchmark.

This data loader covers binary classification and regression datasets in the
GLUE benchmark. It includes 7 binary classification tasks:

   * Corpus of Linguistic Acceptability (COLA)
   * The Stanford Sentiment Treebank (SST2)
   * Microsoft Research Paraphrase Corpus (MRPC)
   * Quora Question Pairs (QQP)
   * Question NLI (QNLI)
   * Recognizing Textual Entailment (RTE)
   * Winograd NLI (WNLI),

and one regression task:

   * Semantic Textual Similarity Benchmark (STSb)

It does not include 3-way classification tasks, e.g., MNLI.

See https://gluebenchmark.com/ for further detail.


## References:
[1] Alex Wang, Amanpreet Singh, Julian Michael, Felix Hill, Omer Levy,
    Samuel Bowman. GLUE: A Multi-Task Benchmark and Analysis Platform for
    Natural Language Understanding.
    In _Proceedings of the 2018 EMNLP Workshop BlackboxNLP: Analyzing and
    Interpreting Neural Networks for NLP_, 2018.
    https://www.aclweb.org/anthology/W18-5446/
"""

from typing import Any, Dict

import tensorflow as tf
import tensorflow_datasets as tfds

from uncertainty_baselines.datasets import base

_FEATURE_NAME = {
    'glue/cola': ['sentence', None],
    'glue/sst2': ['sentence', None],
    'glue/mrpc': ['sentence1', 'sentence2'],
    'glue/qqp': ['question1', 'question2'],
    'glue/qnli': ['question', 'sentence'],
    'glue/rte': ['sentence1', 'sentence2'],
    'glue/wnli': ['sentence1', 'sentence2'],
    'glue/stsb': ['sentence1', 'sentence2'],
}


class _GLUEDataset(base.BaseDataset):
  """CIFAR dataset builder abstract class."""

  def __init__(self,
               name: str,
               batch_size: int,
               eval_batch_size: int,
               shuffle_buffer_size: int = None,
               num_parallel_parser_calls: int = 64,
               **unused_kwargs: Dict[str, Any]):
    """Create a GLUE tf.data.Dataset builder.

    Args:
      name: the name of this dataset.
      batch_size: the training batch size.
      eval_batch_size: the validation and test batch size.
      shuffle_buffer_size: the number of example to use in the shuffle buffer
        for tf.data.Dataset.shuffle().
      num_parallel_parser_calls: the number of parallel threads to use while
        preprocessing in tf.data.Dataset.map().
    """
    tfds_name = 'glue/' + name
    dataset_info = tfds.builder(tfds_name).info

    num_train_examples = dataset_info.splits['train'].num_examples
    num_validation_examples = dataset_info.splits['validation'].num_examples
    num_test_examples = dataset_info.splits['test'].num_examples

    super(_GLUEDataset, self).__init__(
        name=tfds_name,
        num_train_examples=num_train_examples,
        num_validation_examples=num_validation_examples,
        num_test_examples=num_test_examples,
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=shuffle_buffer_size,
        num_parallel_parser_calls=num_parallel_parser_calls)

  def _read_examples(self, split: base.Split) -> tf.data.Dataset:
    """Creates a dataset to be processed by _create_process_example_fn."""
    if split == base.Split.TEST:
      return tfds.load(self.name, split='test')
    elif split == base.Split.TRAIN:
      return tfds.load(self.name, split='train')
    else:
      return tfds.load(self.name, split='validation')

  def _create_process_example_fn(self, split: base.Split) -> base.PreProcessFn:
    """Create a pre-process function to return labels and sentence tokens."""

    def _example_parser(example: Dict[str, tf.Tensor]) -> Dict[str, Any]:
      """Parse snetences and labels from a serialized tf.train.Example."""
      idx = example['idx']
      label = example['label']

      # Read in sentences.
      text_a_name, text_b_name = _FEATURE_NAME[self.name]
      text_a = example[text_a_name]

      text_b = None
      if text_b_name:
        text_b = example[text_b_name]

      parsed_example = {
          'text_a': text_a,
          'text_b': text_b,
          'labels': label,
          'idx': idx
      }

      return parsed_example

    return _example_parser


class ColaDataset(_GLUEDataset):
  """COLA dataset builder class."""

  def __init__(self, **kwargs):
    super(ColaDataset, self).__init__(name='cola', **kwargs)


class Sst2Dataset(_GLUEDataset):
  """SST2 dataset builder class."""

  def __init__(self, **kwargs):
    super(Sst2Dataset, self).__init__(name='sst2', **kwargs)


class MrpcDataset(_GLUEDataset):
  """MRPC dataset builder class."""

  def __init__(self, **kwargs):
    super(MrpcDataset, self).__init__(name='mrpc', **kwargs)


class QqpDataset(_GLUEDataset):
  """QQP dataset builder class."""

  def __init__(self, **kwargs):
    super(QqpDataset, self).__init__(name='qqp', **kwargs)


class StsbDataset(_GLUEDataset):
  """STSb dataset builder class."""

  def __init__(self, **kwargs):
    super(StsbDataset, self).__init__(name='stsb', **kwargs)


class QnliDataset(_GLUEDataset):
  """QNLI dataset builder class."""

  def __init__(self, **kwargs):
    super(QnliDataset, self).__init__(name='qnli', **kwargs)


class RteDataset(_GLUEDataset):
  """RTE dataset builder class."""

  def __init__(self, **kwargs):
    super(RteDataset, self).__init__(name='rte', **kwargs)


class WnliDataset(_GLUEDataset):
  """WNLI dataset builder class."""

  def __init__(self, **kwargs):
    super(WnliDataset, self).__init__(name='wnli', **kwargs)


GlueDatasets = {
    'glue/cola': ColaDataset,
    'glue/sst2': Sst2Dataset,
    'glue/mrpc': MrpcDataset,
    'glue/qqp': QqpDataset,
    'glue/qnli': QnliDataset,
    'glue/rte': RteDataset,
    'glue/wnli': WnliDataset,
    'glue/stsb': StsbDataset,
}



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Model getter utility."""

import json
import logging
from typing import List

import tensorflow.compat.v2 as tf
from uncertainty_baselines.models import bert
from uncertainty_baselines.models import criteo_mlp
from uncertainty_baselines.models import resnet20
from uncertainty_baselines.models import resnet50
from uncertainty_baselines.models import textcnn
from uncertainty_baselines.models import wide_resnet


def get_model_names() -> List[str]:
  return [
      'bert',
      'criteo_mlp',
      'resnet20',
      'resnet50',
      'textcnn',
      'wide_resnet',
  ]


# TODO(znado): check into using Keras' deserialization functionality, similar to
# edward2/tensorflow/initializers.py line 772.
def get(
    model_name: str,
    batch_size: int,
    **hyperparameters) -> tf.keras.Model:
  """Gets a model builder by name.

  Args:
    model_name: Name of the model builder class.
    batch_size: the training batch size.
    **hyperparameters: dict of possible kwargs to be passed to the model
      constructor.

  Returns:
    A model builder class with a method .build(split) which can be called to
    get the tf.data.Dataset, which has elements that are a dict with keys
    'features' and 'labels'.

  Raises:
    ValueError: If model_name is unrecognized.
  """
  logging.info(
      'Building model %s with additional kwargs:\n%s',
      model_name,
      json.dumps(hyperparameters, indent=2, sort_keys=True))
  if model_name not in get_model_names():
    raise ValueError('Unrecognized model type: {!r}'.format(model_name))

  if model_name == 'criteo_mlp':
    return criteo_mlp.create_model(batch_size, **hyperparameters)
  if model_name == 'resnet20':
    return resnet20.create_model(batch_size, **hyperparameters)
  if model_name == 'resnet50':
    return resnet50.create_model(batch_size, **hyperparameters)
  if model_name == 'textcnn':
    return textcnn.create_model(batch_size, **hyperparameters)
  if model_name == 'bert':
    return bert.create_model(batch_size, **hyperparameters)
  if model_name == 'wide_resnet':
    return wide_resnet.create_model(batch_size, **hyperparameters)



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""ResNet50 Keras model for ImageNet."""

from typing import Any, Dict, Tuple

import tensorflow.compat.v2 as tf


def _identity_block(
    input_tensor: tf.Tensor,
    kernel_size: int,
    filters: Tuple[int, int, int],
    stage: int,
    block: str,
    batch_norm_momentum: float,
    batch_norm_epsilon: float) -> tf.Tensor:
  """The identity block is the block that has no conv layer at shortcut.

  Args:
    input_tensor: input tensor
    kernel_size: default 3, the kernel size of middle conv layer at main path.
    filters: list of integers, the filters of 3 conv layer at main path.
    stage: integer, current stage label, used for generating layer names.
    block: 'a','b'..., current block label, used for generating layer names.
    batch_norm_momentum: the batch normalization EMA momentum.
    batch_norm_epsilon: the batch normalization epsilon.

  Returns:
      Output tensor for the block.
  """
  filters1, filters2, filters3 = filters
  # TODO(znado): support NCHW data_format.
  bn_axis = 3
  conv_name_base = 'res' + str(stage) + block + '_branch'
  bn_name_base = 'bn' + str(stage) + block + '_branch'

  x = tf.keras.layers.Conv2D(
      filters1,
      (1, 1),
      use_bias=False,
      kernel_initializer='he_normal',
      name=conv_name_base + '2a')(input_tensor)
  x = tf.keras.layers.BatchNormalization(
      axis=bn_axis,
      name=bn_name_base + '2a',
      momentum=batch_norm_momentum,
      epsilon=batch_norm_epsilon)(x)
  x = tf.keras.layers.Activation('relu')(x)

  x = tf.keras.layers.Conv2D(
      filters2,
      kernel_size,
      use_bias=False,
      padding='same',
      kernel_initializer='he_normal',
      name=conv_name_base + '2b')(x)
  x = tf.keras.layers.BatchNormalization(
      axis=bn_axis,
      name=bn_name_base + '2b',
      momentum=batch_norm_momentum,
      epsilon=batch_norm_epsilon)(x)
  x = tf.keras.layers.Activation('relu')(x)

  x = tf.keras.layers.Conv2D(
      filters3,
      (1, 1),
      use_bias=False,
      kernel_initializer='he_normal',
      name=conv_name_base + '2c')(x)
  x = tf.keras.layers.BatchNormalization(
      axis=bn_axis,
      name=bn_name_base + '2c',
      momentum=batch_norm_momentum,
      epsilon=batch_norm_epsilon)(x)

  x = tf.keras.layers.add([x, input_tensor])
  x = tf.keras.layers.Activation('relu')(x)
  return x


def _conv_block(
    input_tensor: tf.Tensor,
    kernel_size: int,
    filters: Tuple[int, int, int],
    stage: int,
    block: str,
    strides: Tuple[int, int] = (2, 2),
    batch_norm_momentum: float = 0.9,
    batch_norm_epsilon: float = 1e-5) -> tf.Tensor:
  """A block that has a conv layer at shortcut.

  Note that from stage 3, the second conv layer at main path is with
  strides=(2, 2) and the shortcut should have strides=(2, 2) as well.

  Args:
    input_tensor: input tensor
    kernel_size: the kernel size of middle conv layer at main path
    filters: list of integers, the filters of 3 conv layer at main path
    stage: integer, current stage label, used for generating layer names
    block: 'a','b'..., current block label, used for generating layer names
    strides: Strides for the second conv layer in the block.
    batch_norm_momentum: the batch normalization EMA momentum.
    batch_norm_epsilon: the batch normalization epsilon.

  Returns:
    Output tensor for the block.
  """
  filters1, filters2, filters3 = filters
  # TODO(znado): support NCHW data_format.
  bn_axis = 3
  conv_name_base = 'res' + str(stage) + block + '_branch'
  bn_name_base = 'bn' + str(stage) + block + '_branch'

  x = tf.keras.layers.Conv2D(
      filters1,
      (1, 1),
      use_bias=False,
      kernel_initializer='he_normal',
      name=conv_name_base + '2a')(input_tensor)
  x = tf.keras.layers.BatchNormalization(
      axis=bn_axis,
      name=bn_name_base + '2a',
      momentum=batch_norm_momentum,
      epsilon=batch_norm_epsilon)(x)
  x = tf.keras.layers.Activation('relu')(x)

  x = tf.keras.layers.Conv2D(
      filters2,
      kernel_size,
      strides=strides,
      padding='same',
      use_bias=False,
      kernel_initializer='he_normal',
      name=conv_name_base + '2b')(x)
  x = tf.keras.layers.BatchNormalization(
      axis=bn_axis,
      name=bn_name_base + '2b',
      momentum=batch_norm_momentum,
      epsilon=batch_norm_epsilon)(x)
  x = tf.keras.layers.Activation('relu')(x)

  x = tf.keras.layers.Conv2D(
      filters3,
      (1, 1),
      use_bias=False,
      kernel_initializer='he_normal',
      name=conv_name_base + '2c')(x)
  x = tf.keras.layers.BatchNormalization(
      axis=bn_axis,
      name=bn_name_base + '2c',
      momentum=batch_norm_momentum,
      epsilon=batch_norm_epsilon)(x)

  shortcut = tf.keras.layers.Conv2D(
      filters3,
      (1, 1),
      use_bias=False,
      strides=strides,
      kernel_initializer='he_normal',
      name=conv_name_base + '1')(input_tensor)
  shortcut = tf.keras.layers.BatchNormalization(
      axis=bn_axis,
      name=bn_name_base + '1',
      momentum=batch_norm_momentum,
      epsilon=batch_norm_epsilon)(shortcut)

  x = tf.keras.layers.add([x, shortcut])
  x = tf.keras.layers.Activation('relu')(x)
  return x


def create_model(
    batch_size: int,
    batch_norm_momentum: float = 0.9,
    batch_norm_epsilon: float = 1e-5,
    **unused_kwargs: Dict[str, Any]) -> tf.keras.Model:
  """Instantiates the ResNet50 architecture.

  Args:
    batch_size: int value of the static per_replica batch size.
    batch_norm_momentum: the batch normalization EMA momentum.
    batch_norm_epsilon: the batch normalization epsilon.

  Returns:
      A Keras model instance that returns the logits for the model.
  """
  # TODO(znado): support NCHW data format.
  bn_axis = 3
  input_layer = tf.keras.layers.Input(
      shape=(224, 224, 3), batch_size=batch_size)
  x = tf.keras.layers.ZeroPadding2D(
      padding=(3, 3),
      name='conv1_pad')(input_layer)
  x = tf.keras.layers.Conv2D(
      64,
      (7, 7),
      use_bias=False,
      strides=(2, 2),
      padding='valid',
      kernel_initializer='he_normal',
      name='conv1')(x)
  x = tf.keras.layers.BatchNormalization(
      axis=bn_axis,
      name='bn_conv1',
      momentum=batch_norm_momentum,
      epsilon=batch_norm_epsilon)(x)
  x = tf.keras.layers.Activation('relu')(x)
  x = tf.keras.layers.MaxPooling2D((3, 3), strides=(2, 2), padding='same')(x)

  x = _conv_block(
      x,
      kernel_size=3,
      filters=(64, 64, 256),
      stage=2,
      block='a',
      strides=(1, 1),
      batch_norm_momentum=batch_norm_momentum,
      batch_norm_epsilon=batch_norm_epsilon)
  x = _identity_block(
      x, 3, (64, 64, 256), 2, 'b', batch_norm_momentum, batch_norm_epsilon)
  x = _identity_block(
      x, 3, (64, 64, 256), 2, 'c', batch_norm_momentum, batch_norm_epsilon)

  x = _conv_block(
      x,
      kernel_size=3,
      filters=(128, 128, 512),
      stage=3,
      block='a',
      batch_norm_momentum=batch_norm_momentum,
      batch_norm_epsilon=batch_norm_epsilon)
  x = _identity_block(
      x, 3, (128, 128, 512), 3, 'b', batch_norm_momentum, batch_norm_epsilon)
  x = _identity_block(
      x, 3, (128, 128, 512), 3, 'c', batch_norm_momentum, batch_norm_epsilon)
  x = _identity_block(
      x, 3, (128, 128, 512), 3, 'd', batch_norm_momentum, batch_norm_epsilon)

  x = _conv_block(
      x,
      kernel_size=3,
      filters=(256, 256, 1024),
      stage=4,
      block='a',
      batch_norm_momentum=batch_norm_momentum,
      batch_norm_epsilon=batch_norm_epsilon)
  x = _identity_block(
      x, 3, (256, 256, 1024), 4, 'b', batch_norm_momentum, batch_norm_epsilon)
  x = _identity_block(
      x, 3, (256, 256, 1024), 4, 'c', batch_norm_momentum, batch_norm_epsilon)
  x = _identity_block(
      x, 3, (256, 256, 1024), 4, 'd', batch_norm_momentum, batch_norm_epsilon)
  x = _identity_block(
      x, 3, (256, 256, 1024), 4, 'e', batch_norm_momentum, batch_norm_epsilon)
  x = _identity_block(
      x, 3, (256, 256, 1024), 4, 'f', batch_norm_momentum, batch_norm_epsilon)

  x = _conv_block(
      x,
      kernel_size=3,
      filters=(512, 512, 2048),
      stage=5,
      block='a',
      batch_norm_momentum=batch_norm_momentum,
      batch_norm_epsilon=batch_norm_epsilon)
  x = _identity_block(
      x, 3, (512, 512, 2048), 5, 'b', batch_norm_momentum, batch_norm_epsilon)
  x = _identity_block(
      x, 3, (512, 512, 2048), 5, 'c', batch_norm_momentum, batch_norm_epsilon)

  x = tf.keras.layers.GlobalAveragePooling2D(name='avg_pool')(x)
  x = tf.keras.layers.Dense(
      1000,
      kernel_initializer=tf.keras.initializers.RandomNormal(stddev=0.01))(x)
  return tf.keras.Model(input_layer, x, name='resnet50')



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""ImageNet dataset builder.

We have an option to use a percent of the training dataset as a validation set,
and treat the original validation set as the test set. This is similar to what
is also done in the NeurIPS uncertainty benchmark paper
https://arxiv.org/abs/1906.02530 (which used (100 / 1024)% as a validation set).
"""

from typing import Any, Dict
import tensorflow.compat.v2 as tf
import tensorflow_datasets as tfds
from uncertainty_baselines.datasets import base
from uncertainty_baselines.datasets import inception_preprocessing


class ImageNetDataset(base.BaseDataset):
  """ImageNet dataset builder class."""

  def __init__(
      self,
      batch_size: int,
      eval_batch_size: int,
      validation_percent: float = 0.0,
      shuffle_buffer_size: int = None,
      num_parallel_parser_calls: int = 64,
      **unused_kwargs: Dict[str, Any]):
    """Create an ImageNet tf.data.Dataset builder.

    Args:
      batch_size: the training batch size.
      eval_batch_size: the validation and test batch size.
      validation_percent: the percent of the training set to use as a validation
        set.
      shuffle_buffer_size: the number of example to use in the shuffle buffer
        for tf.data.Dataset.shuffle().
      num_parallel_parser_calls: the number of parallel threads to use while
        preprocessing in tf.data.Dataset.map().
    """
    num_train_examples = 1281167
    num_validation_examples = int(num_train_examples * validation_percent)
    num_train_examples -= num_validation_examples
    super(ImageNetDataset, self).__init__(
        name='imagenet',
        num_train_examples=num_train_examples,
        num_validation_examples=num_validation_examples,
        num_test_examples=50000,
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=shuffle_buffer_size,
        num_parallel_parser_calls=num_parallel_parser_calls)

  def _read_examples(self, split: base.Split) -> tf.data.Dataset:
    """Read ImageNet examples. We use the original 'validation' set as test."""
    # TODO(znado): rewrite this to use tfrecords and/or tar.gz files, see
    # cl/295280383 and https://github.com/tensorflow/datasets/issues/1739.
    if split == base.Split.TRAIN:
      if self._num_validation_examples == 0:
        train_split = 'train'
      else:
        train_split = tfds.core.ReadInstruction(
            'train', to=-self._num_validation_examples, unit='abs')
      return tfds.load('imagenet2012:5.0.0', split=train_split)
    elif split == base.Split.VAL:
      if self._num_validation_examples == 0:
        raise ValueError(
            'No validation set provided. Set `validation_percent > 0.0` to '
            'take a subset of the training set as validation.')
      val_split = tfds.core.ReadInstruction(
          'train', from_=-self._num_validation_examples, unit='abs')
      return tfds.load('imagenet2012:5.0.0', split=val_split)
    elif split == base.Split.TEST:
      return tfds.load('imagenet2012', split='validation')

  def _create_process_example_fn(self, split: base.Split) -> base.PreProcessFn:
    """Create a pre-process function to return images in [0, 1]."""

    def _example_parser(example: Dict[str, tf.Tensor]) -> Dict[str, tf.Tensor]:
      """Preprocesses ImageNet image Tensors using inception_preprocessing."""
      # `preprocess_image` returns images in [-1, 1].
      image = inception_preprocessing.preprocess_image(
          example['image'],
          height=224,
          width=224,
          is_training=self._is_training(split))
      # Rescale to [0, 1].
      image = (image + 1.0) / 2.0

      label = tf.cast(example['label'], tf.int32)
      parsed_example = {
          'features': image,
          'labels': label,
      }
      if 'file_name' in example:
        parsed_example['file_name'] = example['file_name']
      return parsed_example

    return _example_parser



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Tests for uncertainty_baselines.datasets.jigsaw_toxic_comment."""

from absl.testing import parameterized
import tensorflow.compat.v2 as tf

import uncertainty_baselines as ub
from uncertainty_baselines.datasets import base
from uncertainty_baselines.datasets import jigsaw_toxic_comment

ub_split = ub.datasets.base.Split


class JigsawToxicCommentTest(tf.test.TestCase, parameterized.TestCase):

  @parameterized.named_parameters(('Train', ub_split.TRAIN),
                                  ('Test', ub_split.TEST))
  def testDatasetSize(self, split):
    batch_size = 9
    eval_batch_size = 5
    dataset_builder = ub.datasets.JigsawToxicCommentDataset(
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=20)
    dataset = dataset_builder.build(split).take(1)
    element = next(iter(dataset))
    features = element['features']
    labels = element['labels']

    expected_batch_size = (
        batch_size if split == ub_split.TRAIN else eval_batch_size)
    expected_feature_length = jigsaw_toxic_comment._FEATURE_LENGTH
    expected_label_length = jigsaw_toxic_comment._LABEL_LENGTH

    feature_batch_size, feature_length = features.shape
    label_batch_size, label_length = labels.shape
    self.assertEqual(feature_batch_size, expected_batch_size)
    self.assertEqual(label_batch_size, expected_batch_size)
    self.assertEqual(feature_length, expected_feature_length)
    self.assertEqual(label_length, expected_label_length)

  def testValidSplitError(self):
    val_split = base.Split.VAL
    zero_val_percent = 0.

    dataset_builder = ub.datasets.JigsawToxicCommentDataset(
        batch_size=32, eval_batch_size=32,
        validation_percent=zero_val_percent,
        shuffle_buffer_size=32)

    with self.assertRaises(ValueError):
      dataset_builder.build(val_split)


if __name__ == '__main__':
  tf.test.main()



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for uncertainty_baselines.models.bert."""

import tensorflow as tf
import uncertainty_baselines as ub

from official.nlp.bert import configs


class BERTTest(tf.test.TestCase):

  def testCreateModel(self):
    """Testing if a BERT model can be configured correctly from BertConfig.

    A BERT-base encoder includes 10 embedding layer ops, 12 hidden layers and
    2 output layer ops. Therefore the created model is expected to contain 12
    layers.
    """
    num_classes = 100
    max_seq_length = 512
    # A standard BERT-base config
    bert_config_dict = configs.BertConfig(
        vocab_size=30521,
        hidden_size=768,
        num_hidden_layers=12,
        num_attention_heads=12,
        intermediate_size=3072,
        hidden_act='gelu',
        hidden_dropout_prob=0.1,
        attention_probs_dropout_prob=0.1,
        max_position_embeddings=512,
        type_vocab_size=16,
        initializer_range=0.02).to_dict()

    _, bert_encoder = ub.models.BERTBuilder(num_classes, max_seq_length,
                                            **bert_config_dict)
    self.assertLen(bert_encoder.layers, 24)


if __name__ == '__main__':
  tf.test.main()



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for uncertainty_baselines.datasets.glue."""

from absl.testing import parameterized

import tensorflow as tf

from uncertainty_baselines.datasets import base
from uncertainty_baselines.datasets import glue


class GlueTest(tf.test.TestCase, parameterized.TestCase):

  @parameterized.named_parameters(('Train', base.Split.TRAIN),
                                  ('Validation', base.Split.VAL),
                                  ('Test', base.Split.TEST))
  def testDatasetSize(self, split):
    batch_size = 9
    eval_batch_size = 5
    for _, dataset_class in glue.GlueDatasets.items():
      dataset_builder = dataset_class(
          batch_size=batch_size,
          eval_batch_size=eval_batch_size,
          shuffle_buffer_size=20)
      dataset = dataset_builder.build(split).take(1)
      element = next(iter(dataset))
      text_a = element['text_a']
      labels = element['labels']

      expected_batch_size = (
          batch_size if split == base.Split.TRAIN else eval_batch_size)
      feature_shape = text_a.shape[0]
      labels_shape = labels.shape[0]

      self.assertEqual(feature_shape, expected_batch_size)
      self.assertEqual(labels_shape, expected_batch_size)

  @parameterized.named_parameters(('Train', base.Split.TRAIN),
                                  ('Validation', base.Split.VAL),
                                  ('Test', base.Split.TEST))
  def testTextbIsNone(self, split):
    batch_size = 9
    eval_batch_size = 5
    for dataset_name, dataset_class in glue.GlueDatasets.items():
      dataset_builder = dataset_class(
          batch_size=batch_size,
          eval_batch_size=eval_batch_size,
          shuffle_buffer_size=20)
      dataset = dataset_builder.build(split).take(1)
      element = next(iter(dataset))
      text_b = element['text_b']

      if dataset_name in ('glue/cola', 'glue/sst2'):
        self.assertIsNone(text_b)
      else:
        self.assertIsNotNone(text_b)


if __name__ == '__main__':
  tf.test.main()



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Uncertainty baseline training datasets."""

from uncertainty_baselines.datasets import inception_preprocessing
from uncertainty_baselines.datasets.base import BaseDataset
from uncertainty_baselines.datasets.cifar import Cifar100Dataset
from uncertainty_baselines.datasets.cifar import Cifar10Dataset
from uncertainty_baselines.datasets.clinc_intent import ClincIntentDetectionDataset
from uncertainty_baselines.datasets.criteo import CriteoDataset
from uncertainty_baselines.datasets.datasets import get
from uncertainty_baselines.datasets.glue import ColaDataset
from uncertainty_baselines.datasets.glue import MrpcDataset
from uncertainty_baselines.datasets.glue import QnliDataset
from uncertainty_baselines.datasets.glue import QqpDataset
from uncertainty_baselines.datasets.glue import RteDataset
from uncertainty_baselines.datasets.glue import Sst2Dataset
from uncertainty_baselines.datasets.glue import WnliDataset
from uncertainty_baselines.datasets.imagenet import ImageNetDataset
from uncertainty_baselines.datasets.jigsaw_toxic_comment import JigsawToxicCommentDataset
from uncertainty_baselines.datasets.mnist import MnistDataset
from uncertainty_baselines.datasets.mnli import MnliDataset
from uncertainty_baselines.datasets.svhn import SvhnDataset
from uncertainty_baselines.datasets.test_utils import DatasetTest



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Collection of utilities for running on different device types."""

from typing import Union
import tensorflow.compat.v2 as tf


_Strategy = Union[
    tf.distribute.MirroredStrategy, tf.distribute.experimental.TPUStrategy]


def get_strategy(master: str, use_tpu: bool) -> _Strategy:
  """Gets a strategy to run locally on CPU or on a fleet of TPUs.

  Args:
    master: A string of the master TPU to run on. Ignored if use_tpu=False.
    use_tpu: Whether or not to use TPU or CPU.

  Returns:
    A TPUStrategy if using TPUs, or a MirroredStrategy if not.
  """
  if use_tpu:
    cluster_resolver = tf.distribute.cluster_resolver.TPUClusterResolver(
        tpu=master)
    tf.config.experimental_connect_to_cluster(cluster_resolver)
    tf.tpu.experimental.initialize_tpu_system(cluster_resolver)
    tpu_strategy = tf.distribute.experimental.TPUStrategy(cluster_resolver)
    return tpu_strategy

  return tf.distribute.MirroredStrategy()



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""SVHN dataset builder."""

from typing import Any, Dict

import tensorflow.compat.v2 as tf
import tensorflow_datasets as tfds
from uncertainty_baselines.datasets import base


class SvhnDataset(base.BaseDataset):
  """SVHN dataset builder class."""

  def __init__(
      self,
      batch_size: int,
      eval_batch_size: int,
      shuffle_buffer_size: int = None,
      num_parallel_parser_calls: int = 64,
      **unused_kwargs: Dict[str, Any]):
    """Create a SVHN tf.data.Dataset builder.

    Args:
      batch_size: the training batch size.
      eval_batch_size: the validation and test batch size.
      shuffle_buffer_size: the number of example to use in the shuffle buffer
        for tf.data.Dataset.shuffle().
      num_parallel_parser_calls: the number of parallel threads to use while
        preprocessing in tf.data.Dataset.map().
    """
    _, info = tfds.load('svhn_cropped', with_info=True)
    super(SvhnDataset, self).__init__(
        name='svhn_cropped',
        num_train_examples=50000,
        num_validation_examples=info.splits['train'].num_examples - 50000,
        num_test_examples=info.splits['test'].num_examples,
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=shuffle_buffer_size,
        num_parallel_parser_calls=num_parallel_parser_calls)

  def _read_examples(self, split: base.Split) -> tf.data.Dataset:
    """We use the original 'validation' set as test."""
    if split == base.Split.TRAIN:
      train_split = tfds.core.ReadInstruction(
          'train', to=-self._num_validation_examples, unit='abs')
      return tfds.load('svhn_cropped', split=train_split)
    elif split == base.Split.VAL:
      val_split = tfds.core.ReadInstruction(
          'train', from_=-self._num_validation_examples, unit='abs')
      return tfds.load('svhn_cropped', split=val_split)
    elif split == base.Split.TEST:
      return tfds.load('svhn_cropped', split='test')
    else:
      raise ValueError(
          'Invalid dataset split in _read_examples: {}'.format(split))

  def _create_process_example_fn(self, split: base.Split) -> base.PreProcessFn:
    del split

    def _example_parser(example: Dict[str, tf.Tensor]) -> Dict[str, tf.Tensor]:
      """A pre-process function to return images in [0, 1]."""
      image = example['image']
      image = tf.image.convert_image_dtype(image, tf.float32)
      label = tf.cast(example['label'], tf.int32)
      parsed_example = {
          'features': image,
          'labels': label,
      }
      return parsed_example

    return _example_parser



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""TextCNN model.

A standard 1D CNN model for sentence classification.

## References
[1]: Yoon Kim. Convolutional Neural Networks for Sentence Classification.
     In _Empirical Methods in Natural Language Processing_, 2014.
     https://www.aclweb.org/anthology/D14-1181/
"""
from typing import Any, Dict, Iterator

import numpy as np
import tensorflow.compat.v2 as tf


def _embedding_block(inputs: tf.Tensor,
                     vocab_size: int,
                     feature_size: int,
                     embed_size: int,
                     premade_embedding_arr: np.ndarray = None) -> tf.Tensor:
  """Creates an embedding layer converting sparse text input into dense vector.

  Args:
    inputs: (tf.Tensor) Input sentence in token indices format, shape
      (batch_size, feature_size).
    vocab_size: (int) Static size of vocabulary.
    feature_size: (int) Static size of input feature.
    embed_size: (int) Static size of hidden dimension of the embedding output.
    premade_embedding_arr: (np.ndarray or None) Pre-made word embedding in numpy
      array format, shape (vocab_size, embed_size). If None then perform random
      initialization for word embedding.

  Raises:
    (ValueError): If shape of premade_embedding_arr is not
      (vocab_size, embed_size).

  Returns:
    (tf.Tensor) shape (batch_size, feature_size, embed_size).
  """
  # Make initializer.
  if premade_embedding_arr is not None:
    premade_vocab_size, premade_embed_size = premade_embedding_arr.shape
    if premade_vocab_size != vocab_size or premade_embed_size != embed_size:
      raise ValueError('"premade_embedding_arr" should have size ({}, {}). '
                       'Observed ({}, {})'.format(vocab_size, embed_size,
                                                  premade_vocab_size,
                                                  premade_embed_size))
    embed_init = tf.keras.initializers.Constant(premade_embedding_arr)
  else:
    embed_init = tf.keras.initializers.RandomUniform(minval=-1, maxval=1)

  # Define layer.
  embedding_layer = tf.keras.layers.Embedding(
      vocab_size,
      embed_size,
      input_length=feature_size,
      embeddings_initializer=embed_init,
      name='embedding')
  return embedding_layer(inputs)


def _conv_pooled_block(inputs: tf.Tensor, num_filters: int, filter_size: int,
                       feature_size: int, embed_size: int) -> tf.Tensor:
  """Creates an 1D convolutional layer with max pooling over sentence positions.

  Args:
    inputs: (tf.Tensor) Input tensor, shape (batch_size, feature_size,
      embed_size).
    num_filters: (int) Number of filters to apply to input.
    filter_size: (int) Static size of the convolutional filter.
    feature_size: (int) Static size of input feature.
    embed_size: (int) Static size of hidden dimension of the text embedding.

  Returns:
    (tf.Tensor) shape (batch_size, 1, num_filters).
  """
  filter_shape = (filter_size, embed_size)
  max_pool_shape = (feature_size - filter_size + 1, 1)

  conv_layer = tf.keras.layers.Conv2D(
      num_filters,
      filter_shape,
      strides=(1, 1),
      padding='valid',
      data_format='channels_last',
      activation='relu',
      kernel_initializer='glorot_normal',
      bias_initializer=tf.keras.initializers.constant(0.1),
      name='convolution_{:d}'.format(filter_size))

  # Max pooling over sentence positions for each filter.
  maxpool_layer = tf.keras.layers.MaxPool2D(
      pool_size=max_pool_shape,
      strides=(1, 1),
      padding='valid',
      data_format='channels_last',
      name='max_pooling_{:d}'.format(filter_size))

  conv = conv_layer(inputs)
  return maxpool_layer(conv)


def create_model(
    batch_size: int,
    num_classes: int,
    feature_size: int,
    vocab_size: int,
    embed_size: int = 300,
    num_filters: int = 128,
    filter_sizes: Iterator[int] = (3, 4, 5),
    dropout_rate: float = 0.2,
    l2_weight: float = 0.001,
    premade_embedding_arr: np.ndarray = None,
    **unused_kwargs: Dict[str, Any]) -> tf.keras.models.Model:
  """Builds TextCNN model.

  Args:
    batch_size: (int) value of the static per_replica batch size.
    num_classes: (int) Number of output classes.
    feature_size: (int) Static size of input feature.
    vocab_size: (int) Static size of vocabulary.
    embed_size: (int) Static size of hidden dimension of the embedding output.
    num_filters: (int) Number of filters to apply to input.
    filter_sizes: (iterable) A iterable object of int (i.e., list or tuple)
      specifying the sizes of the convolutional filters.
    dropout_rate: (float) Fraction of the convolutional output units to drop.
    l2_weight: (float) Strength of L2 regularization for weights in the output
      dense layer.
    premade_embedding_arr: (np.ndarray) Pre-made word embedding in numpy array
      format, shape (vocab_size, embed_size).

  Returns:
    (tf.keras.Model) The TextCNN model.
  """
  inputs = tf.keras.Input(shape=(feature_size,), batch_size=batch_size)

  # Prepare word embedding.
  embed = _embedding_block(
      inputs,
      vocab_size,
      feature_size,
      embed_size,
      premade_embedding_arr=premade_embedding_arr)
  embed = tf.keras.layers.Reshape(
      (feature_size, embed_size, 1), name='add_channel')(embed)

  # Evaluate and gather conv layer output for each filter size.
  pool_outputs = []
  for filter_size in filter_sizes:
    pool = _conv_pooled_block(embed, num_filters, filter_size, feature_size,
                              embed_size)
    pool_outputs.append(pool)

  pool_outputs = tf.keras.layers.concatenate(
      pool_outputs, axis=-1, name='concatenate')

  # Flatten and apply dropout.
  flat_outputs = tf.keras.layers.Flatten(data_format='channels_last',
                                         name='flatten')(pool_outputs)
  flat_outputs = tf.keras.layers.Dropout(dropout_rate,
                                         name='dropout')(flat_outputs)

  # Dense output.
  dense_output_layer = tf.keras.layers.Dense(
      num_classes,
      activation=None,
      kernel_initializer='glorot_normal',
      bias_initializer=tf.keras.initializers.constant(0.1),
      kernel_regularizer=tf.keras.regularizers.l2(l2_weight),
      bias_regularizer=tf.keras.regularizers.l2(l2_weight),
      name='dense_output')
  outputs = dense_output_layer(flat_outputs)

  return tf.keras.Model(inputs=inputs, outputs=outputs, name='textcnn')



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""TF Keras model for an MLP for Criteo, from arxiv.org/abs/1906.02530."""

from typing import Any, Dict, List, Tuple
import tensorflow.compat.v2 as tf

from uncertainty_baselines import datasets


# Model hyperparameters.
_NUM_EMBED_DIMS = [
    3, 9, 29, 11, 17, None, 14, 4, None, 12, 19, 24, 29, None, 13, 25, None,
    8, 29, None, 22, None, None, 31, None, 29,
]
_NUM_HAS_BUCKETS = [
    1373, 2148, 4847, 9781, 396, 28, 3591, 2798, 14, 7403, 2511, 5598, 9501,
    46, 4753, 4056, 23, 3828, 5856, 12, 4226, 23, 61, 3098, 494, 5087,
]
_LAYER_SIZES = [2572, 1454, 1596]


def _make_input_layers(batch_size: int) -> Dict[str, tf.keras.layers.Input]:
  """Defines an input layer for tf.keras model with int32 and string dtypes."""
  out = {}
  for idx in range(1, datasets.criteo.NUM_TOTAL_FEATURES + 1):
    if idx <= datasets.criteo.NUM_INT_FEATURES:
      dtype = tf.int32
    else:
      dtype = tf.string
    name = datasets.criteo.feature_name(idx)
    out[name] = tf.keras.layers.Input(
        [], batch_size=batch_size, dtype=dtype, name=name)
  return out


# In order to get better typing we would need to be able to reference the
# FeatureColumn class defined here, but it is not exported:
# https://github.com/tensorflow/tensorflow/blob/v2.1.0/tensorflow/python/feature_column/feature_column_v2.py#L2121.
def _make_feature_columns() -> Tuple[List[Any], List[Any]]:
  """Build feature_columns for converting features to a dense vector."""
  categorical_feature_columns = []
  categorical_indices = range(
      datasets.criteo.NUM_INT_FEATURES + 1,
      datasets.criteo.NUM_TOTAL_FEATURES + 1)
  for idx in categorical_indices:
    name = datasets.criteo.feature_name(idx)
    cat_idx = idx - datasets.criteo.NUM_INT_FEATURES - 1
    num_buckets = _NUM_HAS_BUCKETS[cat_idx]
    num_embed_dims = _NUM_EMBED_DIMS[cat_idx]

    hash_col = tf.feature_column.categorical_column_with_hash_bucket(
        name, num_buckets)
    if num_embed_dims:
      cat_col = tf.feature_column.embedding_column(hash_col, num_embed_dims)
    else:
      cat_col = tf.feature_column.indicator_column(hash_col)
    categorical_feature_columns.append(cat_col)

  integer_feature_columns = []
  for idx in range(1, datasets.criteo.NUM_INT_FEATURES + 1):
    name = datasets.criteo.feature_name(idx)
    integer_feature_columns.append(tf.feature_column.numeric_column(name))
  return integer_feature_columns, categorical_feature_columns


def create_model(
    batch_size: int,
    **unused_kwargs: Dict[str, Any]) -> tf.keras.models.Model:
  """Creates a tf.keras.Model fully connected model for Criteo."""
  integer_feature_columns, categorical_feature_columns = _make_feature_columns()
  input_layer = _make_input_layers(batch_size)
  integer_features = tf.keras.layers.DenseFeatures(
      integer_feature_columns)(input_layer)
  categorical_features = tf.keras.layers.DenseFeatures(
      categorical_feature_columns)(input_layer)
  x = tf.concat([integer_features, categorical_features], axis=-1)
  x = tf.keras.layers.BatchNormalization()(x)
  for size in _LAYER_SIZES:
    x = tf.keras.layers.Dense(size, activation='relu')(x)
  logits = tf.keras.layers.Dense(1)(x)

  return tf.keras.models.Model(
      inputs=input_layer, outputs=logits, name='criteo_mlp')



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Data loader for the Multi-Genre Natural Language Inference (MNLI) dataset.

MNLI corpus is a crowd-sourced collection of 433k sentence pairs annotated with
textual entailment labels (3 classes: entailment, contradiction and neutral).
The corpus covers a range of genres of spoken and written text from 5 domains:
fiction, government, telephone, travel, slate (i.e., popular magazine).

It also contains ~20k evaluation data from out-of-domain genre (9/11,
face-to-face, letters, oup, verbatim). This evaluation dataset can be used for
evaluating model robustness under distribution shift and for out-of-domain
detection.

See https://cims.nyu.edu/~sbowman/multinli/ and corpus paper for further detail.

## References:

[1] Adina Williams, Nikita Nangia, Samuel Bowman.
    A Broad-Coverage Challenge Corpus for Sentence Understanding through
    Inference.
    In _Proceedings of the 2018 Conference of the North American Chapter of
    the Association for Computational Linguistics_, 2018.
    https://www.aclweb.org/anthology/N18-1101/
"""

from typing import Any, Dict

import tensorflow as tf
import tensorflow_datasets as tfds

from uncertainty_baselines.datasets import base


class MnliDataset(base.BaseDataset):
  """Multi-NLI dataset builder class."""

  def __init__(self,
               batch_size: int,
               eval_batch_size: int,
               shuffle_buffer_size: int = None,
               num_parallel_parser_calls: int = 64,
               mode: str = 'matched',
               **unused_kwargs: Dict[str, Any]):
    """Create a GLUE tf.data.Dataset builder.

    Args:
      batch_size: the training batch size.
      eval_batch_size: the validation and test batch size.
      shuffle_buffer_size: the number of example to use in the shuffle buffer
        for tf.data.Dataset.shuffle().
      num_parallel_parser_calls: the number of parallel threads to use while
        preprocessing in tf.data.Dataset.map().
      mode: Type of data to import. If mode = "matched", import the in-domain
        data (glue/mnli_matched). If mode = "mismatched", import the
        out-of-domain data (glue/mnli_mismatched).
    """
    if mode not in ('matched', 'mismatched'):
      raise ValueError('"mode" must be either "matched" or "mismatched".'
                       'Got {}'.format(mode))
    self.mode = mode
    self.validation_split_name = 'validation_' + mode
    self.test_split_name = 'test_' + mode

    tfds_name = 'glue/mnli'
    dataset_info = tfds.builder(tfds_name).info

    num_train_examples = dataset_info.splits[
        'train'].num_examples if mode == 'matched' else 0
    num_validation_examples = dataset_info.splits[
        self.validation_split_name].num_examples
    num_test_examples = dataset_info.splits[self.test_split_name].num_examples

    super(MnliDataset, self).__init__(
        name=tfds_name,
        num_train_examples=num_train_examples,
        num_validation_examples=num_validation_examples,
        num_test_examples=num_test_examples,
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=shuffle_buffer_size,
        num_parallel_parser_calls=num_parallel_parser_calls)

  def _read_examples(self, split: base.Split) -> tf.data.Dataset:
    """Creates a dataset to be processed by _create_process_example_fn."""
    if split == base.Split.TEST:
      return tfds.load(self.name, split=self.test_split_name)
    elif split == base.Split.TRAIN:
      if self.mode == 'mismatched':
        raise ValueError('No training data for mismatched domains.')
      return tfds.load(self.name, split='train')
    else:
      return tfds.load(self.name, split=self.validation_split_name)

  def _create_process_example_fn(self, split: base.Split) -> base.PreProcessFn:
    """Create a pre-process function to return labels and sentence tokens."""

    def _example_parser(example: Dict[str, tf.Tensor]) -> Dict[str, Any]:
      """Parse sentences and labels from a serialized tf.train.Example."""
      idx = example['idx']
      label = example['label']
      text_a = example['premise']
      text_b = example['hypothesis']

      parsed_example = {
          'text_a': text_a,
          'text_b': text_b,
          'labels': label,
          'idx': idx
      }

      return parsed_example

    return _example_parser



"""Uncertainty Baselines.

See more details in the
[`README.md`](https://github.com/google/uncertainty-baselines).
"""

import os
import sys

from setuptools import find_packages
from setuptools import setup

# To enable importing version.py directly, we add its path to sys.path.
version_path = os.path.join(os.path.dirname(__file__), 'uncertainty_baselines')
sys.path.append(version_path)
from version import __version__  # pylint: disable=g-import-not-at-top

setup(
    name='uncertainty_baselines',
    version=__version__,
    description='Uncertainty Baselines',
    author='Uncertainty Baselines Team',
    author_email='znado@google.com',
    url='http://github.com/google/uncertainty-baselines',
    license='Apache 2.0',
    packages=find_packages(),
    install_requires=[
        'absl-py>=0.5.0',
        'numpy>=1.7',
        'tensorflow>=2.2',
        'tensorflow-datasets>=1.3.0',
        'tf-models-official'
    ],
    extras_require={
        'tf-nightly': ['tf-nightly'],
        'tests': ['pylint>=1.9.0'],
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: Apache Software License',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
    ],
    keywords='probabilistic programming tensorflow machine learning',
)



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Tests for Criteo."""

from absl.testing import parameterized
import tensorflow.compat.v2 as tf
import uncertainty_baselines as ub
from uncertainty_baselines.datasets import base


class CriteoDatasetTest(tf.test.TestCase, parameterized.TestCase):

  @parameterized.named_parameters(
      ('Train', base.Split.TRAIN),
      ('Validation', base.Split.VAL),
      ('Test', base.Split.TEST))
  def testDatasetSize(self, split):
    batch_size = 9
    eval_batch_size = 5
    dataset_builder = ub.datasets.CriteoDataset(
        batch_size=batch_size,
        eval_batch_size=eval_batch_size,
        shuffle_buffer_size=20)
    dataset = dataset_builder.build(split).take(1)
    element = next(iter(dataset))
    features = element['features']
    labels = element['labels']

    expected_batch_size = (
        batch_size if split == base.Split.TRAIN else eval_batch_size)
    features_length = len(features)
    feature_shape = features[list(features)[0]].shape
    labels_shape = labels.shape
    self.assertEqual(feature_shape, (expected_batch_size,))
    self.assertEqual(features_length, 39)
    self.assertEqual(labels_shape, (expected_batch_size,))


if __name__ == '__main__':
  tf.test.main()



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""TF Keras definition for Resnet-20 for CIFAR."""

from typing import Any, Dict

import tensorflow.compat.v2 as tf


def _resnet_layer(
    inputs: tf.Tensor,
    num_filters: int = 16,
    kernel_size: int = 3,
    strides: int = 1,
    use_activation: bool = True,
    use_norm: bool = True,
    l2_weight: float = 1e-4) -> tf.Tensor:
  """2D Convolution-Batch Normalization-Activation stack builder.

  Args:
    inputs: input tensor from input image or previous layer.
    num_filters: Conv2D number of filters.
    kernel_size: Conv2D square kernel dimensions.
    strides: Conv2D square stride dimensions.
    use_activation: whether or not to use a non-linearity.
    use_norm: whether to include normalization.
    l2_weight: the L2 regularization coefficient to use for the convolution
      kernel regularizer.

  Returns:
      Tensor output of this layer.
  """
  kernel_regularizer = None
  if l2_weight:
    kernel_regularizer = tf.keras.regularizers.l2(l2_weight)
  conv_layer = tf.keras.layers.Conv2D(
      num_filters,
      kernel_size=kernel_size,
      strides=strides,
      padding='same',
      kernel_initializer='he_normal',
      kernel_regularizer=kernel_regularizer)

  x = conv_layer(inputs)
  x = tf.keras.layers.BatchNormalization()(x) if use_norm else x
  x = tf.keras.layers.ReLU()(x) if use_activation is not None else x
  return x


def create_model(
    batch_size: int,
    l2_weight: float = 0.0,
    **unused_kwargs: Dict[str, Any]) -> tf.keras.models.Model:
  """Resnet-20 v1, takes (32, 32, 3) input and returns logits of shape (10,)."""
  # TODO(znado): support NCHW data format.
  input_layer = tf.keras.layers.Input(
      shape=(32, 32, 3), batch_size=batch_size)
  depth = 20
  num_filters = 16
  num_res_blocks = int((depth - 2) / 6)

  x = _resnet_layer(
      inputs=input_layer,
      num_filters=num_filters,
      l2_weight=l2_weight)
  for stack in range(3):
    for res_block in range(num_res_blocks):
      strides = 1
      if stack > 0 and res_block == 0:
        strides = 2
      y = _resnet_layer(
          inputs=x,
          num_filters=num_filters,
          strides=strides,
          l2_weight=l2_weight)
      y = _resnet_layer(
          inputs=y,
          num_filters=num_filters,
          use_activation=False,
          l2_weight=l2_weight)
      if stack > 0 and res_block == 0:
        x = _resnet_layer(
            inputs=x,
            num_filters=num_filters,
            kernel_size=1,
            strides=strides,
            use_activation=False,
            use_norm=False,
            l2_weight=l2_weight)
      x = tf.keras.layers.add([x, y])
      x = tf.keras.layers.ReLU()(x)
    num_filters *= 2

  x = tf.keras.layers.AveragePooling2D(pool_size=8)(x)
  x = tf.keras.layers.Flatten()(x)
  logits = tf.keras.layers.Dense(10, kernel_initializer='he_normal')(x)
  return tf.keras.models.Model(
      inputs=input_layer, outputs=logits, name='resnet20')



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Tests for uncertainty_baselines.optimizers."""

import tensorflow.compat.v2 as tf
import uncertainty_baselines as ub


class OptimizersTest(tf.test.TestCase):

  def testAdamW(self):
    weight_decay = 1e-3
    optimizer = ub.optimizers.get(
        optimizer_name='adam',
        learning_rate=0.1,
        weight_decay=weight_decay,
        learning_rate_schedule='constant',
        beta_1=0.9,
        epsilon=1e-1)
    shape = (7,)
    initial_value = tf.ones(shape)
    test_var = tf.Variable(name='test_var', initial_value=initial_value)
    zeros_update = [(tf.zeros(shape), test_var)]
    optimizer.apply_gradients(zeros_update)
    # Because we gave a gradient of 0, the only update to the variable should be
    # from the weight decay.
    self.assertAllClose(initial_value - weight_decay, test_var)

  def testResNet50LearningRateSchedule(self):
    base_learning_rate = 1.0
    steps_per_epoch = 2
    schedule = ub.optimizers.get_learning_rate_schedule(
        schedule_name='resnet50',
        base_learning_rate=base_learning_rate,
        steps_per_epoch=steps_per_epoch)
    actual_learning_rates = [schedule(step) for step in range(200)]
    expected_learning_rates = (
        [base_learning_rate * 1e0] * (40 * steps_per_epoch + 1) +
        [base_learning_rate * 1e-1] * 20 * steps_per_epoch +
        [base_learning_rate * 1e-2] * 20 * steps_per_epoch +
        [base_learning_rate * 1e-3] * 10 * steps_per_epoch +
        [base_learning_rate * 5e-4] * (10 * steps_per_epoch - 1))
    self.assertAllClose(actual_learning_rates, expected_learning_rates)


if __name__ == '__main__':
  tf.test.main()



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Tests for ResNet50."""

import tensorflow.compat.v2 as tf
import uncertainty_baselines as ub


class ResNet50Test(tf.test.TestCase):

  def testCreateModel(self):
    batch_size = 31
    model = ub.models.ResNet50Builder(batch_size)
    logits = model(tf.random.uniform((batch_size, 224, 224, 3)))
    self.assertEqual(logits.shape, (batch_size, 1000))


if __name__ == '__main__':
  tf.test.main()



# coding=utf-8
# Copyright 2020 The Uncertainty Baselines Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Bidirectional Encoder Representations from Transformers (BERT) model.

## References
[1]: Jacob Devlin, Ming-Wei Chang, Kenton Lee, Kristina Toutanova.
     BERT: Pre-training of Deep Bidirectional Transformers for Language
     Understanding.
     In _Proceedings of NAACL-HLT_, 2019.
     https://www.aclweb.org/anthology/N19-1423
"""
import json
from typing import Any, Dict, Tuple, List

import tensorflow as tf
from official.nlp import optimization
from official.nlp.bert import bert_models
from official.nlp.bert import configs



def create_config(config_dir: str) -> Any:
  """Load a BERT config object from directory."""
  with tf.io.gfile.GFile(config_dir) as config_file:
    return json.load(config_file)


def create_feature_and_label(inputs: Dict[str, Any],
                             max_seq_length: int) -> Tuple[List[Any], Any]:
  """Creates features and labels for a BERT model."""
  input_token_ids = inputs['features']
  labels = inputs['labels']
  num_tokens = inputs['num_tokens']

  input_mask = tf.sequence_mask(num_tokens, max_seq_length, dtype=tf.int32)
  type_id = tf.zeros_like(input_mask)
  features = [input_token_ids, input_mask, type_id]

  return features, labels


def create_optimizer(
    initial_lr: float,
    steps_per_epoch: int,
    epochs: int,
    warmup_proportion: float,
    end_lr: float = 0.0,
    optimizer_type: str = 'adamw') -> tf.keras.optimizers.Optimizer:
  """Creates a BERT optimizer with learning rate schedule."""
  num_train_steps = steps_per_epoch * epochs
  num_warmup_steps = int(num_train_steps * warmup_proportion)
  return optimization.create_optimizer(
      initial_lr,
      num_train_steps,
      num_warmup_steps,
      end_lr=end_lr,
      optimizer_type=optimizer_type)


def create_model(
    num_classes: int, max_seq_length: int, initializer_range: float,
    hidden_dropout_prob: float,
    **bert_kwargs: Dict[str, Any]) -> Tuple[tf.keras.Model, tf.keras.Model]:
  """BERT classifier model in functional API style.

  Construct a Keras model for predicting `num_labels` outputs from an input with
  maximum sequence length `max_seq_length`.

  Args:
    num_classes: (int) the number of classes.
    max_seq_length: (int) the maximum input sequence length.
    initializer_range: (float) The stdev of the truncated_normal_initializer for
      initializing all weight matrices.
    hidden_dropout_prob: (float) The dropout probability for all fully connected
      layers in the embeddings, encoder, and pooler.
    **bert_kwargs: Additional arguements to the BertConfig.

  Returns:
    Combined prediction model (words, mask, type) -> (one-hot labels)
    BERT sub-model (words, mask, type) -> (bert_outputs)
  """
  final_layer_initializer = tf.keras.initializers.TruncatedNormal(
      stddev=initializer_range)
  bert_config = configs.BertConfig(
      initializer_range=initializer_range,
      hidden_dropout_prob=hidden_dropout_prob,
      **bert_kwargs)

  bert_encoder = bert_models.get_transformer_encoder(
      bert_config, max_seq_length, output_range=1)

  # initializer
  final_layer_initializer = tf.keras.initializers.TruncatedNormal(
      stddev=initializer_range)

  # build model
  inputs = bert_encoder.inputs
  _, cls_output = bert_encoder(inputs)
  cls_output = tf.keras.layers.Dropout(rate=hidden_dropout_prob)(cls_output)

  # build output
  outputs = tf.keras.layers.Dense(
      num_classes,
      activation=None,
      kernel_initializer=final_layer_initializer,
      name='predictions/transform/logits')(
          cls_output)

  # construct model
  bert_classifier = tf.keras.Model(inputs=inputs, outputs=outputs)

  return bert_classifier, bert_encoder
